#include "scenario.h"

void LoadMap(Map *map, const std::string& filename)
{
	struct Local
	{
		static std::string getStringNode(DataNode* parent, const std::string& name)
		{
			DataNode* n = parent->getChild(name);
			return n->getString();
		}

		static int getIntNode(DataNode* parent, const std::string& name)
		{
			return atoi(getStringNode(parent, name).c_str());
		}
	};
	PHYSFS_file *fp=PHYSFS_openRead(filename.c_str());
	DataNode *rootNode=ReadDoc(fp);
	PHYSFS_close(fp);
	DataNode *realRoot = rootNode->getChild("fft-map");
	DataNode *infoNode = realRoot->getChild("information");
	std::string title=Local::getStringNode(infoNode, "title");

	DataNode* objectsNode = realRoot->getChild("objects");

	DataNodeList mapobjects = objectsNode->getChildren("mapobject");
	DataNodeList::iterator iter;
	if (!mapobjects.empty())
	{
		for (iter = mapobjects.begin(); iter != mapobjects.end(); iter++)
		{
			std::string fpkname = Local::getStringNode(*iter, "fpk");
			float leastfpk = atof(Local::getStringNode(*iter, "fpkversion").c_str());
			float currentfpk = FPK_versionByName(fpkname);
			if (currentfpk<leastfpk)
			{
				char disp[300];
				sprintf(disp, "Scenario requires version %.2f of %s\nVersion found: %.2f", leastfpk, fpkname.c_str(), currentfpk);
				FPK_Invalid(fpkname, disp);
			}
			MapObject *object=new MapObject();
			std::string name = Local::getStringNode(*iter, "name");
			int x = Local::getIntNode(*iter, "x");
			int y = Local::getIntNode(*iter, "y");
			std::string imagefn = Local::getStringNode(*iter, "image");
			int mmr = Local::getIntNode(*iter, "mmr");
			int mmg = Local::getIntNode(*iter, "mmg");
			int mmb = Local::getIntNode(*iter, "mmb");
			object->Init(x,y,imagefn);
			object->SetMMR(mmr);
			object->SetMMG(mmg);
			object->SetMMB(mmb);
			object->SetName(name);
			map->AddToObjects(object);
		}
	}
	DataNodeList actobjects = objectsNode->getChildren("actobject");
	if (!actobjects.empty())
	{
		for (iter = actobjects.begin(); iter != actobjects.end(); iter++)
		{
			int count=0;
			if ((*iter)->hasChild("count")) count=Local::getIntNode(*iter, "count");
			int i;
			std::string fpkname = Local::getStringNode(*iter, "fpk");
			std::string fpk=FPK_ByName(fpkname);
			float leastfpk = atof(Local::getStringNode(*iter, "fpkversion").c_str());
			float currentfpk = FPK_versionByName(fpkname);
			if (currentfpk<leastfpk)
			{
				char disp[300];
				sprintf(disp, "Scenario requires version %.2f of %s\nVersion found: %.2f", leastfpk, fpkname.c_str(), currentfpk);
				FPK_Invalid(fpkname, disp);
			}
			std::string name = Local::getStringNode(*iter, "name");
			int x = Local::getIntNode(*iter, "x");
			int y = Local::getIntNode(*iter, "y");
			std::string imagefn = Local::getStringNode(*iter, "image");
			int mmr = Local::getIntNode(*iter, "mmr");
			int mmg = Local::getIntNode(*iter, "mmg");
			int mmb = Local::getIntNode(*iter, "mmb");
			int credits = Local::getIntNode(*iter, "credits");
			int speed=0;
			int orbitx=0;
			int orbity=0;
			int distance=0;
			if (count)
			{
				for (i=0; i<count; i++)
				{
					if ((*iter)->hasChild("speed"))
					{
						if ((*iter)->getChild("speed")->hasChild("random"))
						{
							int val1=Local::getIntNode((*iter)->getChild("speed")->getChild("random"), "val1");
							int val2=Local::getIntNode((*iter)->getChild("speed")->getChild("random"), "val2");
							speed=(rand()%(val2-val1))+val1;
						}
						else
						{
							speed=Local::getIntNode(*iter, "speed");
						}
					}
					if ((*iter)->hasChild("orbit"))
					{
						orbitx=Local::getIntNode(*iter, "x");
						orbity=Local::getIntNode(*iter, "y");
						if ((*iter)->getChild("orbit")->getChild("distance")->hasChild("random"))
						{
							int val1=Local::getIntNode((*iter)->getChild("orbit")->getChild("distance")->getChild("random"), "val1");
							int val2=Local::getIntNode((*iter)->getChild("orbit")->getChild("distance")->getChild("random"), "val2");
							distance=(rand()%(val2-val1))+val1;
						}
						else
						{
							distance=Local::getIntNode((*iter)->getChild("orbit"), "distance");
						}
					}
					Unit *object=new Unit();
					object->Init(x,y,imagefn,fpk);
					object->SetName(name);
					object->SetSpeed(speed);
					object->SetMMR(mmr);
					object->SetMMG(mmg);
					object->SetMMB(mmb);
					object->SetCredits((unsigned int)credits);
					if (orbitx||orbity||distance)
					{
						object->SetOrbit(true);
						object->Orbit(orbitx,orbity,distance);
					}
					object->FreeImages();
					map->AddToActiveObjects(object);
				}
			}
			else
			{
				if ((*iter)->hasChild("speed"))
				{
					if ((*iter)->getChild("speed")->hasChild("random"))
					{
						int val1=Local::getIntNode((*iter)->getChild("speed")->getChild("random"), "val1");
						int val2=Local::getIntNode((*iter)->getChild("speed")->getChild("random"), "val2");
						speed=(rand()%(val2-val1))+val1;
					}
					else
					{
						speed=Local::getIntNode(*iter, "speed");
					}
				}
				if ((*iter)->hasChild("orbit"))
				{
					orbitx=Local::getIntNode(*iter, "x");
					orbity=Local::getIntNode(*iter, "y");
					if ((*iter)->getChild("orbit")->getChild("distance")->hasChild("random"))
					{
						int val1=Local::getIntNode((*iter)->getChild("orbit")->getChild("distance")->getChild("random"), "val1");
						int val2=Local::getIntNode((*iter)->getChild("orbit")->getChild("distance")->getChild("random"), "val2");
						distance=(rand()%(val2-val1))+val1;
					}
					else
					{
						distance=Local::getIntNode((*iter)->getChild("orbit"), "distance");
					}
				}
				Unit *object=new Unit();
				object->Init(x,y,imagefn,fpk);
				object->SetName(name);
				object->SetSpeed(speed);
				object->SetMMR(mmr);
				object->SetMMG(mmg);
				object->SetMMB(mmb);
				if (orbitx||orbity||distance)
				{
					object->SetOrbit(true);
					object->Orbit(orbitx,orbity,distance);
				}
				object->FreeImages();
				map->AddToActiveObjects(object);
			}
		}
	}
	DataNodeList aiships = objectsNode->getChildren("aiship");
	if (!aiships.empty())
	{
		for (iter = aiships.begin(); iter != aiships.end(); iter++)
		{
			std::string fpkname = Local::getStringNode(*iter, "fpk");
			std::string fpk=FPK_ByName(fpkname);
			float leastfpk = atof(Local::getStringNode(*iter, "fpkversion").c_str());
			float currentfpk = FPK_versionByName(fpkname);
			if (currentfpk<leastfpk)
			{
				char disp[300];
				sprintf(disp, "Scenario requires version %.2f of %s\nVersion found: %.2f", leastfpk, fpkname.c_str(), currentfpk);
				FPK_Invalid(fpkname, disp);
			}
			Unit *object=new Unit();
			std::string name = Local::getStringNode(*iter, "name");
			int x = Local::getIntNode(*iter, "x");
			int y = Local::getIntNode(*iter, "y");
			std::string imagefn = Local::getStringNode(*iter, "image");
			int mmr = Local::getIntNode(*iter, "mmr");
			int mmg = Local::getIntNode(*iter, "mmg");
			int mmb = Local::getIntNode(*iter, "mmb");
			int credits = Local::getIntNode(*iter, "credits");
			int cr=0;
			int cg=0;
			int cb=0;
			if ((*iter)->hasChild("color"))
			{
				cr=Local::getIntNode((*iter)->getChild("color"), "r");
				cg=Local::getIntNode((*iter)->getChild("color"), "g");
				cb=Local::getIntNode((*iter)->getChild("color"), "b");
			}
			int maxspeed = Local::getIntNode(*iter, "maxspeed");
			int hullhitmax = Local::getIntNode(*iter, "hullhitmax");
			std::string saistate = Local::getStringNode(*iter, "aistate");
			int aistate=0;
			if (!strcmp(saistate.c_str(),"AI_STATE_STOP")) aistate=AI_STATE_STOP;
			else if (!strcmp(saistate.c_str(),"AI_STATE_RUN")) aistate=AI_STATE_RUN;
			else if (!strcmp(saistate.c_str(),"AI_STATE_ATTACK")) aistate=AI_STATE_ATTACK;
			else if (!strcmp(saistate.c_str(),"AI_STATE_LOOK")) aistate=AI_STATE_LOOK;
			else if (!strcmp(saistate.c_str(),"AI_STATE_IGNORE")) aistate=AI_STATE_IGNORE;
			else if (!strcmp(saistate.c_str(),"AI_STATE_WAITLOOK")) aistate=AI_STATE_WAITLOOK;
			object->Init(x,y,imagefn,fpk);
			object->SetName(name);
			object->SetMMR(mmr);
			object->SetMMG(mmg);
			object->SetMMB(mmb);
			object->SetCredits((unsigned int)credits);
			object->SetMaxSpeed(maxspeed);
			object->SetHullHitMax(hullhitmax);
			object->SetAIState(aistate);
			object->SetAI(Local::getStringNode(*iter, "aiscript"));
			if (cr||cg||cb) object->Colorize(cr,cg,cb);
			map->AddToAIUnits(object);
		}
	}
	DataNodeList stations = objectsNode->getChildren("station");
	if (!stations.empty())
	{
		for (iter = stations.begin(); iter != stations.end(); iter++)
		{
			std::string fpkname = Local::getStringNode(*iter, "fpk");
			float leastfpk = atof(Local::getStringNode(*iter, "fpkversion").c_str());
			float currentfpk = FPK_versionByName(fpkname);
			if (currentfpk<leastfpk)
			{
				char disp[300];
				sprintf(disp, "Scenario requires version %.2f of %s\nVersion found: %.2f", leastfpk, fpkname.c_str(), currentfpk);
				FPK_Invalid(fpkname, disp);
			}
			Station *object=new Station();
			std::string name = Local::getStringNode(*iter, "name");
			int x = Local::getIntNode(*iter, "x");
			int y = Local::getIntNode(*iter, "y");
			std::string imagefn = Local::getStringNode(*iter, "image");
			int mmr = Local::getIntNode(*iter, "mmr");
			int mmg = Local::getIntNode(*iter, "mmg");
			int mmb = Local::getIntNode(*iter, "mmb");
			object->Init(x,y,imagefn);
			object->SetMMR(mmr);
			object->SetMMG(mmg);
			object->SetMMB(mmb);
			object->SetName(name);
			if ((*iter)->hasChild("shop"))
			{
				if ((*iter)->getChild("shop")->hasChild("buy"))
				{
					DataNodeList items=(*iter)->getChild("shop")->getChild("buy")->getChildren("item");
					DataNodeList::iterator iter2;
					if (!items.empty())
					{
						for (iter2 = items.begin(); iter2 != items.end(); iter2++)
						{
							if (!((*iter2)->hasChild("type"))) SCN_Invalid(filename,"Map file contains an item with no type");
							if (!((*iter2)->hasChild("name"))) SCN_Invalid(filename,"Map file contains an item with no name");
							std::string fpk;
							if ((*iter2)->hasChild("dispimage") || (*iter2)->hasChild("image"))
							{
								std::string fpkname = Local::getStringNode(*iter2, "fpk");
								fpk=FPK_ByName(fpkname);
								float leastfpk = atof(Local::getStringNode(*iter2, "fpkversion").c_str());
								float currentfpk = FPK_versionByName(fpkname);
								if (currentfpk<leastfpk)
								{
									char disp[300];
									sprintf(disp, "Scenario requires version %.2f of %s\nVersion found: %.2f", leastfpk, fpkname.c_str(), currentfpk);
									FPK_Invalid(fpkname, disp);
								}
							}
							__shopitem item;
							item.type=Local::getStringNode(*iter2, "type");
							item.fpk=fpk;
							if ((*iter2)->hasChild("baytype")) item.baytype=Local::getStringNode(*iter2, "baytype");
							if ((*iter2)->hasChild("holdweapon")) item.holdweapon=Local::getStringNode(*iter2, "holdweapon");
							if ((*iter2)->hasChild("amount")) item.amount=Local::getIntNode(*iter2, "amount");
							else item.amount=0;
							if ((*iter2)->hasChild("holdamount")) item.holdamount=Local::getIntNode(*iter2, "holdamount");
							else item.holdamount=0;
							if ((*iter2)->hasChild("cargounits")) item.cargounits=Local::getIntNode(*iter2, "cargounits");
							else item.cargounits=0;
							item.name=Local::getStringNode(*iter2, "name");
							if ((*iter2)->hasChild("description")) item.desc=Local::getStringNode(*iter2, "description");
							if ((*iter2)->hasChild("price"))
							{
								item.baseprice=0;
								item.price=Local::getIntNode(*iter2, "price");
							}
							else if ((*iter2)->hasChild("baseprice"))
							{
								item.baseprice=Local::getIntNode(*iter2, "baseprice");
								item.price=Local::getIntNode(*iter2, "baseprice");
							}
							else SCN_Invalid(filename,"Map file contains an item with no price or baseprice");
							if ((*iter2)->hasChild("dispimage")) item.dispimage=FPK_LoadIMG(fpk,Local::getStringNode(*iter2, "dispimage"));
							else item.dispimage=0;
							if ((*iter2)->hasChild("image")) item.imagefn=Local::getStringNode(*iter2, "image");
							if ((*iter2)->hasChild("baseimport")) item.baseimport=Local::getIntNode(*iter2, "baseimport");
							else item.baseimport=0;
							if ((*iter2)->hasChild("w") && !strcmp(item.type.c_str(),"bay")) item.bayw=Local::getIntNode(*iter2, "w");
							else item.bayw=0;
							if ((*iter2)->hasChild("l") && !strcmp(item.type.c_str(),"bay")) item.bayl=Local::getIntNode(*iter2, "l");
							else item.bayl=0;
							if ((*iter2)->hasChild("d") && !strcmp(item.type.c_str(),"bay")) item.bayd=Local::getIntNode(*iter2, "d");
							else item.bayd=0;
							object->AddItemToBuyShop(item);
						}
					}
				}
				if ((*iter)->getChild("shop")->hasChild("sell"))
				{
					DataNodeList items=(*iter)->getChild("shop")->getChild("sell")->getChildren("item");
					DataNodeList::iterator iter2;
					if (!items.empty())
					{
						for (iter2 = items.begin(); iter2 != items.end(); iter2++)
						{
							if (!((*iter2)->hasChild("name"))) SCN_Invalid(filename,"Map file contains an item with no name");
							//std::string fpk;
							std::string type=Local::getStringNode(*iter2, "type");
							if (!strcmp(type.c_str(),"cargo"))
							{
								__cargo cargo;
								cargo.type=Local::getStringNode(*iter2, "name");
								if ((*iter2)->hasChild("price"))
								{
									cargo.priceBought=Local::getIntNode(*iter2, "price");
								}
								else if ((*iter2)->hasChild("baseprice"))
								{
									cargo.priceBought=Local::getIntNode(*iter2, "baseprice");
								}
								object->AddSellableCargo(cargo);
							}
						}
					}
				}
				if ((*iter)->hasChild("repairs"))
				{
					if ((*iter)->getChild("repairs")->hasChild("shields"))
					{
						if (!(*iter)->getChild("repairs")->getChild("shields")->hasChild("baseprice")) SCN_Invalid(filename, "shield repairs found, but no baseprice command found");
						unsigned int cost=Local::getIntNode((*iter)->getChild("repairs")->getChild("shields"), "baseprice");
						object->AllowShieldRepairs(cost);
					}
					if ((*iter)->getChild("repairs")->hasChild("hull"))
					{
						if (!(*iter)->getChild("repairs")->getChild("hull")->hasChild("baseprice")) SCN_Invalid(filename, "hull repairs found, but no baseprice command found");
						unsigned int cost=Local::getIntNode((*iter)->getChild("repairs")->getChild("hull"), "baseprice");
						object->AllowHullRepairs(cost);
					}
				}
			}
			map->AddToStations(object);
		}
	}
	StaticAnim *anim=new StaticAnim;
	anim->LoadFrames("gfx/anim/explosion");
	map->AddToAnimations(anim);
	delete rootNode;
}

void Scenario::Init(std::string fn, Map *map)
{
	this->SetMap(map);
	this->ReadOnly(fn);
}
void Scenario::ReadOnly(std::string fn)
{
	victnext=0;
	condnext=0;
	lossnext=0;
	lcondnext=0;
	startx=0;
	starty=0;
	startang=0;
	startspeed=0;
	credits=0;
	startingshipnext=0;
	int i;
	for (i=0; i<50; i++)
	{
		get[i]=false;
		got[i]=false;
	}
	struct Local
	{
		static std::string getStringNode(DataNode* parent, const std::string& name)
		{
			DataNode* n = parent->getChild(name);
			return n->getString();
		}

		static int getIntNode(DataNode* parent, const std::string& name)
		{
			return atoi(getStringNode(parent, name).c_str());
		}
	};
	file=fn;
	PHYSFS_file *fp=PHYSFS_openRead(file.c_str());
	DataNode *rootNode=ReadDoc(fp);
	PHYSFS_close(fp);
	if (!rootNode->hasChild("fft-scn")) SCN_Invalid(file, "File is not a valid Scenario file");
	DataNode *realRoot = rootNode->getChild("fft-scn");
	if (!realRoot->hasChild("information")) SCN_Invalid(file, "information block not found in Scenario file");
	if (!realRoot->getChild("information")->hasChild("title")) SCN_Invalid(file, "title command not found in information block");
	if (!realRoot->getChild("information")->hasChild("description")) SCN_Invalid(file, "description command not found in information block");
	name=Local::getStringNode(realRoot->getChild("information"), "title");
	desc=Local::getStringNode(realRoot->getChild("information"), "description");
	if (!realRoot->hasChild("mapfile")) SCN_Invalid(name, "mapfile command not found in Scenario file");
	mapfn="scenario/";
	mapfn+=Local::getStringNode(realRoot, "mapfile");
	if (!realRoot->hasChild("openmessage")) SCN_Invalid(name, "openmessage command not found in Scenario file");
	openmessage=Local::getStringNode(realRoot, "openmessage");
	if (!realRoot->hasChild("victory")) SCN_Invalid(name, "victory command not found in Scenario file");
	DataNode *victoryNode = realRoot->getChild("victory");
	DataNodeList::iterator iter;
	DataNodeList conditionsets = victoryNode->getChildren("conditionset");
	if (!conditionsets.empty())
	{
		for (iter = conditionsets.begin(); iter != conditionsets.end(); iter++)
		{
			if ((*iter)->hasChild("actobjects"))
			{
				conditionset[victnext]=condnext;
				if ((*iter)->hasChild("meet"))
				{
					meet[victnext]=Local::getIntNode(*iter, "meet");
				}
				else SCN_Invalid(name, "actobjects victory condition has no 'meet' command");
				if ((*iter)->getChild("actobjects")->hasChild("whenmet"))
				{
					get[victnext]=true;
					getwhat[victnext]=Local::getStringNode((*iter)->getChild("actobjects"), "whenmet");
				}
				valvictory[victnext] = "actobjects";
				if ((*iter)->getChild("actobjects")->hasChild("equals"))
				{
					victory[victnext] = Local::getIntNode((*iter)->getChild("actobjects"), "equals");
					compvictory[victnext] = "equals";
					victnext++;
				}
				else if ((*iter)->getChild("actobjects")->hasChild("notequal"))
				{
					victory[victnext] = Local::getIntNode((*iter)->getChild("actobjects"), "notequal");
					compvictory[victnext] = "notequal";
					victnext++;
				}
				else if ((*iter)->getChild("actobjects")->hasChild("lessthan"))
				{
					victory[victnext] = Local::getIntNode((*iter)->getChild("actobjects"), "lessthan");
					compvictory[victnext] = "lessthan";
					victnext++;
				}
				else if ((*iter)->getChild("actobjects")->hasChild("greaterthan"))
				{
					victory[victnext] = Local::getIntNode((*iter)->getChild("actobjects"), "greaterthan");
					compvictory[victnext] = "greaterthan";
					victnext++;
				}
				else if ((*iter)->getChild("actobjects")->hasChild("lessthanorequal"))
				{
					victory[victnext] = Local::getIntNode((*iter)->getChild("actobjects"), "lessthanorequal");
					compvictory[victnext] = "lessthanorequal";
					victnext++;
				}
				else if ((*iter)->getChild("actobjects")->hasChild("greaterthanorequal"))
				{
					victory[victnext] = Local::getIntNode((*iter)->getChild("actobjects"), "greaterthanorequal");
					compvictory[victnext] = "greaterthanorequal";
					victnext++;
				}
				else SCN_Invalid(name, "actobjects victory condition has no valid command");
			}
			if ((*iter)->hasChild("aiships"))
			{
				conditionset[victnext]=condnext;
				if ((*iter)->hasChild("meet"))
				{
					meet[victnext]=Local::getIntNode(*iter, "meet");
				}
				else SCN_Invalid(name, "aiships victory condition has no 'meet' command");
				if ((*iter)->getChild("aiships")->hasChild("whenmet"))
				{
					get[victnext]=true;
					getwhat[victnext]=Local::getStringNode((*iter)->getChild("aiships"), "whenmet");
				}
				valvictory[victnext] = "aiships";
				if ((*iter)->getChild("aiships")->hasChild("equals"))
				{
					victory[victnext] = Local::getIntNode((*iter)->getChild("aiships"), "equals");
					compvictory[victnext] = "equals";
					victnext++;
				}
				else if ((*iter)->getChild("aiships")->hasChild("notequal"))
				{
					victory[victnext] = Local::getIntNode((*iter)->getChild("aiships"), "notequal");
					compvictory[victnext] = "notequal";
					victnext++;
				}
				else if ((*iter)->getChild("aiships")->hasChild("lessthan"))
				{
					victory[victnext] = Local::getIntNode((*iter)->getChild("aiships"), "lessthan");
					compvictory[victnext] = "lessthan";
					victnext++;
				}
				else if ((*iter)->getChild("aiships")->hasChild("greaterthan"))
				{
					victory[victnext] = Local::getIntNode((*iter)->getChild("aiships"), "greaterthan");
					compvictory[victnext] = "greaterthan";
					victnext++;
				}
				else if ((*iter)->getChild("aiships")->hasChild("lessthanorequal"))
				{
					victory[victnext] = Local::getIntNode((*iter)->getChild("aiships"), "lessthanorequal");
					compvictory[victnext] = "lessthanorequal";
					victnext++;
				}
				else if ((*iter)->getChild("aiships")->hasChild("greaterthanorequal"))
				{
					victory[victnext] = Local::getIntNode((*iter)->getChild("aiships"), "greaterthanorequal");
					compvictory[victnext] = "greaterthanorequal";
					victnext++;
				}
				else SCN_Invalid(name, "aiships victory condition has no valid command");
			}
			condnext++;
		}
	}
	else SCN_Invalid(name, "No 'conditionset's found under victory command");
	if (realRoot->hasChild("loss"))
	{
		DataNode *lossNode = realRoot->getChild("loss");
		DataNodeList conditionsets2 = lossNode->getChildren("conditionset");
		if (!conditionsets2.empty())
		{
			for (iter = conditionsets2.begin(); iter != conditionsets2.end(); iter++)
			{
				if ((*iter)->hasChild("actobjects"))
				{
					lconditionset[lossnext]=lcondnext;
					if ((*iter)->hasChild("meet"))
					{
						lmeet[lossnext]=Local::getIntNode(*iter, "meet");
					}
					else SCN_Invalid(name, "actobjects loss condition has no 'meet' command");
					/*
					if ((*iter)->getChild("actobjects")->hasChild("whenmet"))
					{
						get[lossnext]=true;
						getwhat[lossnext]=Local::getStringNode((*iter)->getChild("actobjects"), "whenmet");
					}
					*/
					valloss[lossnext] = "actobjects";
					if ((*iter)->getChild("actobjects")->hasChild("equals"))
					{
						loss[lossnext] = Local::getIntNode((*iter)->getChild("actobjects"), "equals");
						comploss[lossnext] = "equals";
						lossnext++;
					}
					else if ((*iter)->getChild("actobjects")->hasChild("notequal"))
					{
						loss[lossnext] = Local::getIntNode((*iter)->getChild("actobjects"), "notequal");
						comploss[lossnext] = "notequal";
						lossnext++;
					}
					else if ((*iter)->getChild("actobjects")->hasChild("lessthan"))
					{
						loss[lossnext] = Local::getIntNode((*iter)->getChild("actobjects"), "lessthan");
						comploss[lossnext] = "lessthan";
						lossnext++;
					}
					else if ((*iter)->getChild("actobjects")->hasChild("greaterthan"))
					{
						loss[lossnext] = Local::getIntNode((*iter)->getChild("actobjects"), "greaterthan");
						comploss[lossnext] = "greaterthan";
						lossnext++;
					}
					else if ((*iter)->getChild("actobjects")->hasChild("lessthanorequal"))
					{
						loss[lossnext] = Local::getIntNode((*iter)->getChild("actobjects"), "lessthanorequal");
						comploss[lossnext] = "lessthanorequal";
						lossnext++;
					}
					else if ((*iter)->getChild("actobjects")->hasChild("greaterthanorequal"))
					{
						loss[lossnext] = Local::getIntNode((*iter)->getChild("actobjects"), "greaterthanorequal");
						comploss[lossnext] = "greaterthanorequal";
						lossnext++;
					}
					else SCN_Invalid(name, "actobjects loss condition has no valid command");
				}
				if ((*iter)->hasChild("aiships"))
				{
					lconditionset[lossnext]=lcondnext;
					if ((*iter)->hasChild("meet"))
					{
						lmeet[lossnext]=Local::getIntNode(*iter, "meet");
					}
					else SCN_Invalid(name, "aiships loss condition has no 'meet' command");
					/*
					if ((*iter)->getChild("aiships")->hasChild("whenmet"))
					{
						get[lossnext]=true;
						getwhat[lossnext]=Local::getStringNode((*iter)->getChild("aiships"), "whenmet");
					}
					*/
					valloss[lossnext] = "aiships";
					if ((*iter)->getChild("aiships")->hasChild("equals"))
					{
						loss[lossnext] = Local::getIntNode((*iter)->getChild("aiships"), "equals");
						comploss[lossnext] = "equals";
						lossnext++;
					}
					else if ((*iter)->getChild("aiships")->hasChild("notequal"))
					{
						loss[lossnext] = Local::getIntNode((*iter)->getChild("aiships"), "notequal");
						comploss[lossnext] = "notequal";
						lossnext++;
					}
					else if ((*iter)->getChild("aiships")->hasChild("lessthan"))
					{
						loss[lossnext] = Local::getIntNode((*iter)->getChild("aiships"), "lessthan");
						comploss[lossnext] = "lessthan";
						lossnext++;
					}
					else if ((*iter)->getChild("aiships")->hasChild("greaterthan"))
					{
						loss[lossnext] = Local::getIntNode((*iter)->getChild("aiships"), "greaterthan");
						comploss[lossnext] = "greaterthan";
						lossnext++;
					}
					else if ((*iter)->getChild("aiships")->hasChild("lessthanorequal"))
					{
						loss[lossnext] = Local::getIntNode((*iter)->getChild("aiships"), "lessthanorequal");
						comploss[lossnext] = "lessthanorequal";
						lossnext++;
					}
					else if ((*iter)->getChild("aiships")->hasChild("greaterthanorequal"))
					{
						loss[lossnext] = Local::getIntNode((*iter)->getChild("aiships"), "greaterthanorequal");
						comploss[lossnext] = "greaterthanorequal";
						lossnext++;
					}
					else SCN_Invalid(name, "aiships loss condition has no valid command");
				}
				lcondnext++;
			}
		}
		else SCN_Invalid(name, "No 'conditionset's found under loss command");
	}
	if (!realRoot->hasChild("start")) SCN_Invalid(name, "start command not found in Scenario file");
	if (!realRoot->getChild("start")->hasChild("x")) SCN_Invalid(name, "start command does not have x command");
	if (!realRoot->getChild("start")->hasChild("y")) SCN_Invalid(name, "start command does not have y command");
	if (!realRoot->getChild("start")->hasChild("angle")) SCN_Invalid(name, "start command does not have angle command");
	if (!realRoot->getChild("start")->hasChild("speed")) SCN_Invalid(name, "start command does not have speed command");
	if (!realRoot->getChild("start")->hasChild("credits")) SCN_Invalid(name, "start command does not have credits command");
	if (realRoot->getChild("start")->hasChild("disablewarp")) disablewarp=true;
	else disablewarp=false;
	startx=Local::getIntNode(realRoot->getChild("start"), "x");
	starty=Local::getIntNode(realRoot->getChild("start"), "y");
	startang=((Local::getIntNode(realRoot->getChild("start"), "angle"))/5);
	startang=(startang*5)%360;
	startspeed=Local::getIntNode(realRoot->getChild("start"), "speed");
	credits=Local::getIntNode(realRoot->getChild("start"), "credits");
	if (!realRoot->hasChild("startingships")) SCN_Invalid(name, "startingships command not found in Scenario file");
	DataNodeList ships = realRoot->getChild("startingships")->getChildren("ship");
	if (!ships.empty())
	{
		for (iter = ships.begin(); iter != ships.end(); iter++)
		{
			if (!(*iter)->hasChild("name")) SCN_Invalid(name, "ship under starting ships is missing command 'name'");
			if (!(*iter)->hasChild("fpk")) SCN_Invalid(name, "ship under starting ships is missing command 'fpk'");
			startingshipnames[startingshipnext]=Local::getStringNode(*iter, "name");
			startingshipfpks[startingshipnext]=Local::getStringNode(*iter, "fpk");
			startingshipnext++;
		}
	}
	delete rootNode;
}
void Scenario::SetMap(Map *map)
{
	scnmap=map;
}
void Scenario::Startup()
{
	if (disablewarp) scnmap->mainship->DisableWarp();
	else scnmap->mainship->EnableWarp();
}
bool Scenario::CheckVictory()
{
	bool retval=false;
	int met[50];
	int i;
	for (i=0; i<50; i++)
	{
		met[i]=0;
	}
	int bigmet=0;
	int savemet=0;
	if (!victnext && !lossnext)
	{
		fprintf(stderr, "There are no victory conditions or loss conditions!  This should never happen!");
		exit(1);
	}
	else
	{
		for (i=0; i<victnext; i++)
		{
			savemet=met[conditionset[i]];
			if (!strcmp(valvictory[i].c_str(),"actobjects"))
			{
				if (!strcmp(compvictory[i].c_str(),"equals"))
				{
					if(scnmap->GetObjCount()==victory[i]) met[conditionset[i]]++;
				}
				else if (!strcmp(compvictory[i].c_str(),"notequal"))
				{
					if(scnmap->GetObjCount()!=victory[i]) met[conditionset[i]]++;
				}
				else if (!strcmp(compvictory[i].c_str(),"lessthan"))
				{
					if(scnmap->GetObjCount()<victory[i]) met[conditionset[i]]++;
				}
				else if (!strcmp(compvictory[i].c_str(),"greaterthan"))
				{
					if(scnmap->GetObjCount()>victory[i]) met[conditionset[i]]++;
				}
				else if (!strcmp(compvictory[i].c_str(),"lessthanorequal"))
				{
					if(scnmap->GetObjCount()<=victory[i]) met[conditionset[i]]++;
				}
				else if (!strcmp(compvictory[i].c_str(),"greaterthanorequal"))
				{
					if(scnmap->GetObjCount()>=victory[i]) met[conditionset[i]]++;
				}
			}
			if (!strcmp(valvictory[i].c_str(),"aiships"))
			{
				if (!strcmp(compvictory[i].c_str(),"equals"))
				{
					if(scnmap->GetAICount()==victory[i]) met[conditionset[i]]++;
				}
				else if (!strcmp(compvictory[i].c_str(),"notequal"))
				{
					if(scnmap->GetAICount()!=victory[i]) met[conditionset[i]]++;
				}
				else if (!strcmp(compvictory[i].c_str(),"lessthan"))
				{
					if(scnmap->GetAICount()<victory[i]) met[conditionset[i]]++;
				}
				else if (!strcmp(compvictory[i].c_str(),"greaterthan"))
				{
					if(scnmap->GetAICount()>victory[i]) met[conditionset[i]]++;
				}
				else if (!strcmp(compvictory[i].c_str(),"lessthanorequal"))
				{
					if(scnmap->GetAICount()<=victory[i]) met[conditionset[i]]++;
				}
				else if (!strcmp(compvictory[i].c_str(),"greaterthanorequal"))
				{
					if(scnmap->GetAICount()>=victory[i]) met[conditionset[i]]++;
				}
			}
			if (savemet<met[conditionset[i]] && get[i] && !got[i])
			{
				if (!strcmp(getwhat[i].c_str(),"enablewarp")) scnmap->mainship->EnableWarp();
				got[i]=true;
			}
		}
	}
	int j;
	for(i=0; i<condnext; i++)
	{
		for(j=0; j<victnext; j++)
		{
			if (i==conditionset[j])
			{
				if (met[i]>=meet[j]) bigmet++;
			}
		}
	}
	if (bigmet>=condnext) retval=true;
	else retval=false;
	return retval;
}
bool Scenario::CheckLoss()
{
	bool retval=false;
	int lmet[50];
	int i;
	for (i=0; i<50; i++)
	{
		lmet[i]=0;
	}
	int lbigmet=0;
	if (!victnext && !lossnext)
	{
		fprintf(stderr, "There are no victory conditions AND no loss conditions!  This should never happen!");
		exit(1);
	}
	else
	{
		if (lossnext)
		{
			for (i=0; i<lossnext; i++)
			{
				if (!strcmp(valloss[i].c_str(),"actobjects"))
				{
					if (!strcmp(comploss[i].c_str(),"equals"))
					{
						if(scnmap->GetObjCount()==loss[i]) lmet[lconditionset[i]]++;
					}
					else if (!strcmp(comploss[i].c_str(),"notequal"))
					{
						if(scnmap->GetObjCount()!=loss[i]) lmet[lconditionset[i]]++;
					}
					else if (!strcmp(comploss[i].c_str(),"lessthan"))
					{
						if(scnmap->GetObjCount()<loss[i]) lmet[lconditionset[i]]++;
					}
					else if (!strcmp(comploss[i].c_str(),"greaterthan"))
					{
						if(scnmap->GetObjCount()>loss[i]) lmet[lconditionset[i]]++;
					}
					else if (!strcmp(comploss[i].c_str(),"lessthanorequal"))
					{
						if(scnmap->GetObjCount()<=loss[i]) lmet[lconditionset[i]]++;
					}
					else if (!strcmp(comploss[i].c_str(),"greaterthanorequal"))
					{
						if(scnmap->GetObjCount()>=loss[i]) lmet[lconditionset[i]]++;
					}
				}
				if (!strcmp(valloss[i].c_str(),"aiships"))
				{
					if (!strcmp(comploss[i].c_str(),"equals"))
					{
						if(scnmap->GetAICount()==loss[i]) lmet[lconditionset[i]]++;
					}
					else if (!strcmp(comploss[i].c_str(),"notequal"))
					{
						if(scnmap->GetAICount()!=loss[i]) lmet[lconditionset[i]]++;
					}
					else if (!strcmp(comploss[i].c_str(),"lessthan"))
					{
						if(scnmap->GetAICount()<loss[i]) lmet[lconditionset[i]]++;
					}
					else if (!strcmp(comploss[i].c_str(),"greaterthan"))
					{
						if(scnmap->GetAICount()>loss[i]) lmet[lconditionset[i]]++;
					}
					else if (!strcmp(comploss[i].c_str(),"lessthanorequal"))
					{
						if(scnmap->GetAICount()<=loss[i]) lmet[lconditionset[i]]++;
					}
					else if (!strcmp(comploss[i].c_str(),"greaterthanorequal"))
					{
						if(scnmap->GetAICount()>=loss[i]) lmet[lconditionset[i]]++;
					}
				}
			}
			int j;
			for(i=0; i<lcondnext; i++)
			{
				for(j=0; j<lossnext; j++)
				{
					if (i==lconditionset[j])
					{
						if (lmet[i]>=lmeet[j]) lbigmet++;
					}
				}
			}
			if (lbigmet>=lcondnext) retval=true;
			else retval=false;
		}
	}
	return retval;
}
std::string Scenario::GetMapFn()
{
	return mapfn;
}
std::string Scenario::GetName()
{
	return name;
}
std::string Scenario::GetDesc()
{
	return desc;
}
std::string Scenario::GetOpenMessage()
{
	return openmessage;
}
int Scenario::GetStartX()
{
	return startx;
}
int Scenario::GetStartY()
{
	return starty;
}
int Scenario::GetStartAngle()
{
	return startang;
}
int Scenario::GetStartSpeed()
{
	return startspeed;
}
unsigned int Scenario::GetStartCredits()
{
	return credits;
}
bool Scenario::ShipIsAllowed(std::string name, std::string fpk)
{
	int i;
	if (startingshipnext)
	{
		for(i=0; i<startingshipnext; i++)
		{
			if (!strcmp(name.c_str(), startingshipnames[i].c_str()) && !strcmp(fpk.c_str(), FPK_ByName(startingshipfpks[i]).c_str()))
			{
				return true;
			}
		}
	}
	else return true;
	return false;
}

bool IsScenario(char *str)
{
	unsigned int start=strlen(str)-8;
	char temp[9];
	unsigned int i;
	for(i=0; i<strlen(str)-start; i++)
	{
		temp[i]=str[i+start];
	}
	temp[8]='\0';
	if (!strcmp(temp,".fft-scn")) return true;
	else return false;
}
std::string ChooseScenario()
{
	SDL_Event event;
	SDL_Surface *screen=SDL_GetVideoSurface();
	int w=screen->w;
	int h=screen->h;
	GUI *sGUI=new GUI;
	SDL_Color white = { 0xFF, 0xFF, 0xFF, 0 };
	TTF_Font *font=TTF_OpenFont("fonts/arial.ttf", 12);
	TTF_Font *font14=TTF_OpenFont("fonts/arial.ttf", 14);
	char **scnlist;
	char **k;
	std::string retval;
	sGUI->Init();
	sGUI->MakeWindow(30, h-50, 50, 34);
	sGUI->MakeWindow(w/2-60, h-50, 120, 34);
	sGUI->MakeWindow(w-80, h-50, 50, 34);
	int prev=sGUI->MakeButton(35, h-45, 40, "< Prev");
	int select=sGUI->MakeButton(w/2-55, h-45, 110, "Select this Scenario");
	int next=sGUI->MakeButton(w-75, h-45, 40, "Next >");
	int done=0;

	//if there is an order file, order the scenarios accordingly
	const int total=100;
	std::string scen[total];
	std::string name[total];
	std::string desc[total];
	DataNode *rootNode;
	DataNodeList orderedScenarios;
	DataNodeList::iterator iter;
	PHYSFS_file *fp=PHYSFS_openRead("scenario/scn-order.fs");
	int scencount=0;
	if (fp)
	{
		rootNode=ReadDoc(fp);
		orderedScenarios = rootNode->getChild("fft-scnorder")->getChildren("scenario");
		if (!orderedScenarios.empty())
		{
			for (iter = orderedScenarios.begin(); iter != orderedScenarios.end(); iter++)
			{
				Scenario *Scen=new Scenario();
				std::string mapfn="scenario/";
				mapfn+=(*iter)->getString();
				Scen->ReadOnly(mapfn);
				scen[scencount]=mapfn;
				name[scencount]=Scen->GetName();
				desc[scencount]=Scen->GetDesc();
				scencount++;
				delete Scen;
			}
		}
		PHYSFS_close(fp);
	}

	//Find all remaining scenario files
	scnlist=PHYSFS_enumerateFiles("scenario");
	for (k = scnlist; *k != NULL; k++)
	{
		int check=0;
		bool goahead=true;
		if (!orderedScenarios.empty())
		{
			for (iter = orderedScenarios.begin(); iter != orderedScenarios.end(); iter++)
			{
				std::string compfn="scenario/";
				compfn+=(*iter)->getString();
				if (!strcmp(scen[check].c_str(),compfn.c_str())) goahead=false;
				check++;
			}
		}
		if(IsScenario(*k) && goahead)
		{
			Scenario *Scen=new Scenario();
			std::string mapfn="scenario/";
			mapfn+=*k;
			Scen->ReadOnly(mapfn);
			scen[scencount]=mapfn;
			name[scencount]=Scen->GetName();
			desc[scencount]=Scen->GetDesc();
			scencount++;
			delete Scen;
		}
	}
	if (!scencount) SCN_Invalid("List", "Could not find any scenarios");
	int cur=0;
	PHYSFS_freeList(scnlist);
	int getx, gety, click;
	while(!done)
	{
		Slock();
		while (SDL_PollEvent(&event))
		{
			// an event was found
			switch (event.type) {
			// close button clicked
			case SDL_QUIT:
				exit(0);
				break;

			// handle the keyboard
			case SDL_KEYDOWN:
				switch (event.key.keysym.sym) {
					case SDLK_ESCAPE:
					done = 1;
					break;
					default:
						break;
				}
				break;
			case SDL_MOUSEMOTION:
				SDL_GetMouseState(&getx, &gety);
				sGUI->SendMouseMotion(getx,gety);
				break;
			case SDL_MOUSEBUTTONDOWN:
				SDL_GetMouseState(&getx, &gety);
				click = sGUI->SendMouseClick(getx, gety);
				break;
			case SDL_MOUSEBUTTONUP:
				SDL_GetMouseState(&getx, &gety);
				if (click!=-1 && click==sGUI->SendMouseClick(getx, gety))
				{
					SDL_FillRect(screen,0,0);
					centerString(font14, w/2, 20, "Scenario Selection", white);
					centerString(font, w/2, 60, name[cur], white);
					drawString(font, 60, 80, "Description: ", white);
					drawString(font, 80, 100, desc[cur], white);
					sGUI->Draw();
					Sulock();
					SDL_Flip(screen);
					if (click==prev)
					{
						if (cur-1<0) cur=scencount-1;
						else cur--;
					}
					if (click==select)
					{
						retval=scen[cur];
						done=1;
					}
					if (click==next)
					{
						if (cur+1>=scencount) cur=0;
						else cur++;
					}
				}
				sGUI->UpdateButtons();
				break;
			default:
				break;
			}
		}
		SDL_FillRect(screen,0,0);
		centerString(font14, w/2, 20, "Scenario Selection", white);
		centerString(font, w/2, 60, name[cur], white);
		drawString(font, 60, 80, "Description: ", white);
		drawString(font, 80, 100, desc[cur], white);
		sGUI->Draw();
		Sulock();
		SDL_Flip(screen);
	}
	TTF_CloseFont(font);
	TTF_CloseFont(font14);
	delete sGUI;
	delete rootNode;
	return retval;
}
int NewShipOptions(int w, int h, Scenario *Scen)
{
	if (LOG) printf("Inside Ship options\n");
	SDL_Event event;
	SDL_Surface *screen=SDL_GetVideoSurface();
	GUI *sGUI=new GUI;
	SDL_Color white = { 0xFF, 0xFF, 0xFF, 0 };
	TTF_Font *font=TTF_OpenFont("fonts/arial.ttf", 12);
	TTF_Font *font14=TTF_OpenFont("fonts/arial.ttf", 14);
	char **fpklist;
	char **k;
	Uint8 cr[6];
	Uint8 cg[6];
	Uint8 cb[6];
	cr[0]=0;   cg[0]=0;   cb[0]=0; //none
	cr[1]=0;   cg[1]=255; cb[1]=255; //cyan
	cr[2]=0;   cg[2]=255; cb[2]=0;   //green
	cr[3]=103; cg[3]=8;   cb[3]=123; //violet
	cr[4]=255; cg[4]=255; cb[4]=0;   //yellow
	cr[5]=255; cg[5]=255; cb[5]=255; //white
	int numcolors=6;
	sGUI->Init();
	sGUI->MakeWindow(30, h-50, 50, 34);
	sGUI->MakeWindow(w/2-50, h-50, 100, 34);
	sGUI->MakeWindow(w-80, h-50, 50, 34);
	sGUI->MakeWindow(w-120, h/2-45, 90, 34);
	sGUI->MakeWindow(w-120, h/2+11, 90, 34);
	int prev=sGUI->MakeButton(35, h-45, 40, "< Prev");
	int select=sGUI->MakeButton(w/2-45, h-45, 90, "Select this Ship");
	int next=sGUI->MakeButton(w-75, h-45, 40, "Next >");
	int prevcolor=sGUI->MakeButton(w-115, h/2-40, 80, "< Prev Color");
	int nextcolor=sGUI->MakeButton(w-115, h/2+16, 80, "Next Color >");
	int done=0;
	std::string loadfpk;
	int i, j, getx, gety, click, cur, temp, temp2, fpks, max, curfpk, count, curcolor;
	int total[60];
	max=0;
	cur=0;
	curfpk=0;
	count=0;
	curcolor=0;
	FPKShip *curship=new FPKShip(0,0);
	FPKShip *tempship=new FPKShip(0,0);
	fpklist=PHYSFS_enumerateFiles("fpk");
	fpks=FPK_count();
	i=0;
	for (k = fpklist; *k != NULL; k++)
	{
		if(IsFPK(*k))
		{
			total[i]=FPK_countPlayerShips(*k);
			max+=total[i];
			i++;
		}
	}
	PHYSFS_freeList(fpklist);
	loadfpk="base.fpk";
	//make sure the ship that shows up first is the right one!
	temp=0;
	for(i=0; i<fpks; i++)
	{
		for(j=temp; j<total[i]+temp; j++)
		{
			temp2=0;
			while (!temp2)
			{
				tempship->GetFileInfo(i, j-temp);
				if (cur==j && (!Scen->ShipIsAllowed(tempship->GetName(),tempship->GetFPK())))
				{
					if (cur+1>=max) cur=0;
					else cur++;
					i=0;
					j=0;
					temp=0;
				}
				else temp2=1;
			}
			if (cur==j)
			{
				curship->Reload(i, j-temp);
				curfpk=i;
			}
		}
		temp=j;
	}
	fpklist=PHYSFS_enumerateFiles("fpk");
	for (k = fpklist; *k != NULL; k++)
	{
		if(IsFPK(*k))
		{
			if (curfpk==count) loadfpk=*k;
			count++;
		}
	}
	count=0;
	PHYSFS_freeList(fpklist);
	curship->Colorize(cr[curcolor], cg[curcolor], cb[curcolor]);
	while(!done)
	{
		Slock();
		while (SDL_PollEvent(&event))
		{
			// an event was found
			switch (event.type) {
			// close button clicked
			case SDL_QUIT:
				done=-1;
				break;

			// handle the keyboard
			case SDL_KEYDOWN:
				switch (event.key.keysym.sym) {
					case SDLK_ESCAPE:
					done = 1;
					break;
					default:
						break;
				}
				break;
			case SDL_MOUSEMOTION:
				SDL_GetMouseState(&getx, &gety);
				sGUI->SendMouseMotion(getx,gety);
				break;
			case SDL_MOUSEBUTTONDOWN:
				SDL_GetMouseState(&getx, &gety);
				click = sGUI->SendMouseClick(getx, gety);
				break;
			case SDL_MOUSEBUTTONUP:
				SDL_GetMouseState(&getx, &gety);
				if (click!=-1 && click==sGUI->SendMouseClick(getx, gety))
				{
					SDL_FillRect(screen,0,0);
					centerString(font14, w/2, 20, "Ship Selection", white);
					centerString(font, w/2, 60, curship->GetName(), white);
					drawString(font, 20, h-80, "Slots: ", white);
					drawString(font, 60, h-80, curship->GetSSlotCount(), white);
					drawString(font, w-150, 60, "Maximum Warp: ", white);
					drawString(font, w-40, 60, curship->GetMaxWarp(), white);
					drawString(font, w-150, 76, "Cruise Warp: ", white);
					drawString(font, w-40, 76, curship->GetMaxSafeWarp(), white);
					drawString(font, w-150, 92, "Width: ", white);
					drawString(font, w-40, 92, curship->GetSW(), white);
					drawString(font, w-150, 108, "Length: ", white);
					drawString(font, w-40, 108, curship->GetSL(), white);
					drawString(font, w-150, 124, "Depth: ", white);
					drawString(font, w-40, 124, curship->GetSD(), white);
					DrawIMG(curship->GetImage(),w/2-curship->GetImage()->w/2,h/2-curship->GetImage()->h/2);
					if(curship->HasShield()) DrawIMG(curship->GetShield(),w/2-curship->GetShield()->w/2,h/2-curship->GetShield()->h/2);
					sGUI->Draw();
					rectangleRGBA(screen, w-120, h/2-9, w-30, h/2+9, 255, 255, 255, 255);
					if (curcolor>0) boxRGBA(screen, w-119, h/2-8, w-30, h/2+8, cr[curcolor], cg[curcolor], cb[curcolor], 255);
					Sulock();
					SDL_Flip(screen);
					if (click==prev)
					{
						if (cur-1<0) cur=max-1;
						else cur--;
						temp=0;
						for(i=0; i<fpks; i++)
						{
							for(j=temp; j<total[i]+temp; j++)
							{
								temp2=0;
								while (!temp2)
								{
									tempship->GetFileInfo(i, j-temp);
									if (cur==j && (!Scen->ShipIsAllowed(tempship->GetName(),tempship->GetFPK())))
									{
										if (cur-1<0) cur=max-1;
										else cur--;
										i=0;
										j=0;
										temp=0;
									}
									else temp2=1;
								}
								if (cur==j)
								{
									curship->Reload(i, j-temp);
									curfpk=i;
								}
							}
							temp=j;
						}
						fpklist=PHYSFS_enumerateFiles("fpk");
						for (k = fpklist; *k != NULL; k++)
						{
							if(IsFPK(*k))
							{
								if (curfpk==count) loadfpk=*k;
								count++;
							}
						}
						count=0;
						PHYSFS_freeList(fpklist);
						curship->Colorize(cr[curcolor], cg[curcolor], cb[curcolor]);
					}
					if (click==select)
					{
						if (LOG) printf("Setting ship angle to 0\n");
						ship->SetAngle(0);
						if (LOG) printf("Colorizing ship\n");
						ship->Colorize(cr[curcolor], cg[curcolor], cb[curcolor]);
						if (LOG) printf("Getting New ship images\n");
						ship->FPK_NewImages(loadfpk, curship->GetImageFN());
						if (LOG) printf("Setting max speed\n");
						ship->SetMaxSpeed(curship->GetMaxSpeed());
						if (LOG) printf("Setting max warp speed\n");
						ship->SetMaxWarp(curship->GetMaxWarpSpeed());
						if (LOG) printf("Setting max safe warp speed\n");
						ship->SetMaxSafeWarp(curship->GetMaxSafeWarpSpeed());
						if (LOG) printf("Setting turn delay\n");
						ship->SetTurnDelay(curship->GetTurnDelay());
						if (LOG) printf("Setting warp turn delay\n");
						ship->SetWarpTurnDelay(curship->GetWarpTurnDelay());
						if (LOG) printf("Filling slots\n");
						ship->FillSlots(curship->GetSlots(),curship->GetSlotCount());
						if (LOG) printf("Setting shoot point\n");
						ship->SetShootPoint(curship->GetShootX(),curship->GetShootY());
						if (LOG) printf("Setting overheat\n");
						ship->SetOverHeatMax(curship->GetOverHeatMax());
						if (LOG) printf("Adding weapons\n");
						ship->AddWeaponToSlot(101,"base.fpk","gfx/object/bullet.png");
						//ship->AddWeaponToSlot(102,"base.fpk","gfx/object/bullet2.png");
						if (LOG && curship->HasShield()) printf("Adding shield\n");
						if (curship->HasShield()) ship->AddShield(curship->GetShieldFN());
						if (LOG) printf("Ship loaded\n");
						done=1;
					}
					if (click==next)
					{
						if (cur+1>=max) cur=0;
						else cur++;
						temp=0;
						for(i=0; i<fpks; i++)
						{
							for(j=temp; j<total[i]+temp; j++)
							{
								temp2=0;
								while (!temp2)
								{
									tempship->GetFileInfo(i, j-temp);
									if (cur==j && (!Scen->ShipIsAllowed(tempship->GetName(),tempship->GetFPK())))
									{
										if (cur+1>=max) cur=0;
										else cur++;
										i=0;
										j=0;
										temp=0;
									}
									else temp2=1;
								}
								if (cur==j)
								{
									curship->Reload(i, j-temp);
									curfpk=i;
								}
							}
							temp=j;
						}
						fpklist=PHYSFS_enumerateFiles("fpk");
						for (k = fpklist; *k != NULL; k++)
						{
							if(IsFPK(*k))
							{
								if (curfpk==count) loadfpk=*k;
								count++;
							}
						}
						count=0;
						PHYSFS_freeList(fpklist);
						curship->Colorize(cr[curcolor], cg[curcolor], cb[curcolor]);
					}
					if (click==prevcolor)
					{
						if (curcolor-1>=0) curcolor--;
						else curcolor=numcolors-1;
						curship->Colorize(cr[curcolor], cg[curcolor], cb[curcolor]);
					}
					if (click==nextcolor)
					{
						if (curcolor+1<numcolors) curcolor++;
						else curcolor=0;
						curship->Colorize(cr[curcolor], cg[curcolor], cb[curcolor]);
					}
				}
				sGUI->UpdateButtons();
				break;
			default:
				break;
			}
		}
		SDL_FillRect(screen,0,0);
		centerString(font14, w/2, 20, "Ship Selection", white);
		centerString(font, w/2, 60, curship->GetName(), white);
		drawString(font, 20, h-80, "Slots: ", white);
		drawString(font, 60, h-80, curship->GetSSlotCount(), white);
		drawString(font, w-150, 60, "Maximum Warp: ", white);
		drawString(font, w-40, 60, curship->GetMaxWarp(), white);
		drawString(font, w-150, 76, "Cruise Warp: ", white);
		drawString(font, w-40, 76, curship->GetMaxSafeWarp(), white);
		drawString(font, w-150, 92, "Width: ", white);
		drawString(font, w-40, 92, curship->GetSW(), white);
		drawString(font, w-150, 108, "Length: ", white);
		drawString(font, w-40, 108, curship->GetSL(), white);
		drawString(font, w-150, 124, "Depth: ", white);
		drawString(font, w-40, 124, curship->GetSD(), white);
		DrawIMG(curship->GetImage(),w/2-curship->GetImage()->w/2,h/2-curship->GetImage()->h/2);
		if(curship->HasShield()) DrawIMG(curship->GetShield(),w/2-curship->GetShield()->w/2,h/2-curship->GetShield()->h/2);
		sGUI->Draw();
		rectangleRGBA(screen, w-120, h/2-9, w-30, h/2+9, 255, 255, 255, 255);
		if (curcolor>0) boxRGBA(screen, w-119, h/2-8, w-30, h/2+8, cr[curcolor], cg[curcolor], cb[curcolor], 255);
		Sulock();
		SDL_Flip(screen);
	}
	TTF_CloseFont(font);
	TTF_CloseFont(font14);
	delete curship;
	delete tempship;
	delete sGUI;
	return done;
}
void SCN_Invalid(std::string scn, std::string message)
{
	TTF_Font *font=TTF_OpenFont("fonts/arial.ttf", 12);
	SDL_Color white = { 0xFF, 0xFF, 0xFF, 0 };
	SDL_Surface *screen=SDL_GetVideoSurface();
	std::string msg[20];
	char c;
	Uint16 i=0;
	int end=0;
	int done=0;
	std::string msghead;
	c=message[i];
	while(!done) {
		if(c != 0x0A) {
			msg[end] += c;
		}
		else end++;
		i++;
		if (i>=message.size()) done=1;
		c=message[i];
	}
	msghead="Scenario: ";
	msghead+=scn;
	msghead+=" was invalid for the following reasons:";
	Uint32 ticks=SDL_GetTicks();
	SDL_Event event;
	bool printfd=false;
	while (SDL_GetTicks()<ticks+30000)
	{
		while (SDL_PollEvent(&event))
		{
			// an event was found
			switch (event.type) {
			// close button clicked
			case SDL_QUIT:
				exit(1);
				break;
			}
		}
		SDL_FillRect(screen,0,0);
		drawString(font, screen->w/2-200, 20, msghead, white);
		if (LOG && !printfd) printf("%s\n",msghead.c_str());
		for (i=0; i<=end; i++)
		{
			drawString(font, screen->w/2-200, 40+(i*20), msg[i], white);
			if (LOG && !printfd) printf("%s\n",msg[i].c_str());
		}
		drawString(font, screen->w/2-200, 40+((i+1)*20), "This game will autoclose in 30 seconds", white);
		SDL_Flip(screen);
		printfd=true;
	}
	exit(1);
}
