#include "units.h"
#include "gui.h"

Unit::Unit()
{
	Reset();
}
Unit::~Unit()
{
	CleanUp();
}
void Unit::Reset()
{
	int i;
	for (i=0; i<UNITS_NUM_FRAMES; i++)
	{
		mask[i]=0; smask[i]=0; image[i]=0; shield[i]=0;
	}
	dispimg=0;
	for (i=0; i<__MAXSHIPSLOTS; i++)
	{
		slots[i].image=0;
	}
}
void Unit::CleanUp()
{
	int i;
	for (i=0; i<UNITS_NUM_FRAMES; i++)
	{
		if (mask[i]) { bitmask_free(mask[i]); mask[i]=0; }
		if (smask[i]) { bitmask_free(smask[i]); smask[i]=0; }
		if (image[i]) { SDL_FreeSurface(image[i]); image[i]=0; }
		if (shield[i]) { SDL_FreeSurface(shield[i]); shield[i]=0; }
	}
	if (dispimg) { SDL_FreeSurface(dispimg); dispimg=0; }
	for (i=0; i<numslots; i++)
	{
		if (slots[i].image) SDL_FreeSurface(slots[i].image);
	}
	angleinc=5;
	angle=0;
	angle2=0;
	speed=0;
	zoom=1;
	name="";
}
void Unit::Init(int sx, int sy, std::string img, std::string fpk)
{
	int i;
	SDL_Surface *temp;
	angleinc=5;
	angle=0;
	angle2=0;
	speed=0;
	zoom=1;
	name="";
	usingfpk=fpk;
	x=sx;
	y=sy;
	dx=x*1.0;
	dy=y*1.0;
	aiscript="";
	aiwait=SDL_GetTicks();
	aistate=AI_STATE_STOP;
	aiwaitfor=0;
	imgload=img;
	if (!strcmp(usingfpk.c_str()," ")) temp=LoadIMG(imgload);
	else temp=FPK_LoadIMG(usingfpk, imgload);
	dispimg=rotozoomSurface(temp, 0, 0.5, SMOOTH);
	mmr=255;
	mmg=255;
	mmb=0;
	cr=0;
	cg=0;
	cb=0;
	hasShield=false;
	for(i=0; i<UNITS_NUM_FRAMES; i++)
	{
		image[i]=0;
		shield[i]=0;
	}
	for(i=0; i<UNITS_NUM_FRAMES; i++)
	{
		image[i]=rotozoomSurface(temp, 360-i*angleinc, zoom, SMOOTH);
		mask[i]=MakeMask(image[i]);
	}
	warp=false;
	warpenabled=true;
	orbiter=false;
	maxspeed=50;
	maxwarp=50;
	maxsafewarp=50;
	weaponcount=0;
	curweapon=0;
	hullhitmax=1;
	shieldhitmax=1;
	numhullhits=0;
	numshieldhits=0;
	aifire=false;
	credits=0;
	origshootx=0;
	origshooty=0;
	this->AdjustShootPoint();
	overheatcount=0;
	overheatmax=1;
	overheatwait=SDL_GetTicks();
	for(i=0; i<__MAXSHIPSLOTS; i++)
	{
		slots[i].image=0;
	}
	this->ZeroSlots();
	cargounits=0;
	cargofill=0;
	SDL_FreeSurface(temp);
}
void Unit::Display(int sx, int sy)
{
	int ang;
	ang=(int)angle/angleinc;
	int centx, centy, shx, shy;
	centx=sx+(image[ang]->w/2);
	centy=sy+(image[ang]->h/2);
	DrawIMG(image[ang], sx, sy);
	if (hasShield && shield[ang])
	{
		shx=centx-(shield[ang]->w/2);
		shy=centy-(shield[ang]->h/2);
		DrawIMG(shield[ang], shx, shy);
	}
}
void Unit::StatusDisplay(int sx, int sy, int sw, int sh, TTF_Font *font)
{
	char disp[300];
	SDL_Color white = { 0xFF, 0xFF, 0xFF, 0 };
	SDL_Color red = { 0xFF, 0x88, 0x88, 0 };
	SDL_Surface *screen=SDL_GetVideoSurface();
	//Uint8 r, g, b;
	int shipx, shipy;
	boxRGBA(screen, sx, sy, sx+sw, sy+sh, 128,128,128,164);
	if (!warp) sprintf(disp, "Speed: %d", speed);
	if (warp && speed!=maxwarp) sprintf(disp, "Speed: Warp %.2f", SpeedToFWarp(speed));
	if (warp && speed==maxwarp) sprintf(disp, "Speed: Warp %.1f", SpeedToFWarp(speed));
	if (speed<=maxsafewarp) drawString(font, sx+5, sy, disp, white);
	if (speed>maxsafewarp) drawString(font, sx+5, sy, disp, red);
	sprintf(disp, "Sector: (%d,%d)", (int)x/1000, (int)y/1000);
	drawString(font, sx+5, sy+9, disp, white);
	sprintf(disp, "Credits: %d", credits);
	drawString(font, sx+5, sy+18, disp, white);
	int shieldpercent=(int)((((float)shieldhitmax-numshieldhits)/shieldhitmax)*100);
	if (!hasShield) shieldpercent=0;
	int hullpercent=(int)((((float)hullhitmax-numhullhits)/hullhitmax)*100);
	sprintf(disp, "Shields: %d%%", shieldpercent);
	drawString(font, sx+5, sy+27, disp, white);
	sprintf(disp, "Hull Integrity: %d%%", hullpercent);
	drawString(font, sx+5, sy+36, disp, white);
	int enginetemp=(int)((((float)overheatcount)/overheatmax)*100);
	sprintf(disp, "Engine Temp: %d%%", enginetemp);
	if (enginetemp<50) drawString(font, sx+5, sy+45, disp, white);
	if (enginetemp>=50) drawString(font, sx+5, sy+45, disp, red);
	shipx=sx+((sw-dispimg->w)/2);
	shipy=sy+18+((sh-dispimg->h)/2);
	DrawIMG(this->GetCurWeaponSlot().image, sx+(sw/2)-(this->GetCurWeaponSlot().image->w/2), sy+sh-(this->GetCurWeaponSlot().image->h/2)-18);
	drawString(font, sx+5, sy+sh-25, "Weapon:", white);
	int ammo=this->GetCurWeaponSlot().ammoamount;
	if (ammo)
	{
		sprintf(disp, "Ammo: %d", ammo);
		drawString(font, sx+5, sy+sh-15, disp, white);
	}
	DrawIMG(dispimg, shipx, shipy);
}
void Unit::SetSpeed(int sSpeed)
{
	speed=sSpeed;
	if (speed>maxspeed && !warp && !orbiter) speed=maxspeed;
	if (speed>maxwarp && warp && !orbiter) speed=maxwarp;
}
void Unit::SetAngle(int sAngle)
{
	angle=sAngle;
}
void Unit::SetX(int sx)
{
	x=sx;
	dx=x*1.0;
}
void Unit::SetY(int sy)
{
	y=sy;
	dy=y*1.0;
}
void Unit::SetAI(std::string ai)
{
	aiscript=ai;
}
void Unit::SetAIState(int state)
{
	aistate=state;
}
void Unit::Move(int sx, int sy)
{
	double dangle;
	int destangle, dstangle;
	if (!strcmp(aiscript.c_str(),"SIMPLE"))
	{
		dangle=atan2((sy-y),(sx-x));
		destangle=(int)(dangle*(180/PI))+90;
		if (destangle<0) destangle+=360;
		dstangle=destangle-(destangle%5);
		int xd = (sx/1000)-(x/1000);
		int yd = (sy/1000)-(y/1000);
		int distance = (int)sqrt(xd*xd + yd*yd);
		if (Abs((double)distance)>=2 && aistate!=AI_STATE_LOOK) aistate=AI_STATE_WAITLOOK;
		if (SDL_GetTicks()>=aiwait+aiwaitfor)
		{
			switch (aistate)
			{
				case AI_STATE_LOOK:
					if (speed>0) speed-=1;
					else if (speed<0) speed+=1;
					if (dstangle!=angle)
					{
						if (angle<destangle && destangle-angle>=180) this->Rotate(0);
						else if (angle<destangle && destangle-angle>=0) this->Rotate(1);
						else if (angle<destangle && destangle-angle<0) this->Rotate(0);
						else if (angle>destangle && destangle-angle>=-180) this->Rotate(0);
						else if (angle>destangle && destangle-angle<-180) this->Rotate(1);
					}
					aiwaitfor=50;
					aiwait=SDL_GetTicks();
					break;
				case AI_STATE_WAITLOOK:
					if (speed>0) speed-=1;
					else if (speed<0) speed+=1;
					if (dstangle!=angle)
					{
						if (angle<destangle && destangle-angle>=180) this->Rotate(0);
						else if (angle<destangle && destangle-angle>=0) this->Rotate(1);
						else if (angle<destangle && destangle-angle<0) this->Rotate(0);
						else if (angle>destangle && destangle-angle>=-180) this->Rotate(0);
						else if (angle>destangle && destangle-angle<-180) this->Rotate(1);
					}
					if (Abs((double)distance)<=0.5)
					{
						int random=rand()%2;
						if (random) aistate=AI_STATE_ATTACK;
						else aistate=AI_STATE_RUN;
					}
					aiwaitfor=50;
					aiwait=SDL_GetTicks();
					break;
				case AI_STATE_RUN:
					destangle=(destangle+180)%360;
					if (angle<destangle && destangle-angle>=180) this->Rotate(0);
					else if (angle<destangle && destangle-angle>=0) this->Rotate(1);
					else if (angle<destangle && destangle-angle<0) this->Rotate(0);
					else if (angle>destangle && destangle-angle>=-180) this->Rotate(0);
					else if (angle>destangle && destangle-angle<-180) this->Rotate(1);
					if (speed<maxspeed) speed+=1;
					aiwaitfor=200;
					aiwait=SDL_GetTicks();
					break;
				case AI_STATE_ATTACK:
					aiwaitfor=100+rand()%50;
					if (Abs((double)distance)>=0.4 && speed<maxspeed)
					{
						speed+=1;
						aiwaitfor=rand()%50;
					}
					else if (Abs((double)distance)>=2) aistate=AI_STATE_WAITLOOK;
					else if (speed>2 && dstangle==angle) speed-=1;
					else if (speed>1) speed-=1;
					if (dstangle!=angle)
					{
						if (angle<destangle && destangle-angle>=180) this->Rotate(0);
						else if (angle<destangle && destangle-angle>=0) this->Rotate(1);
						else if (angle<destangle && destangle-angle<0) this->Rotate(0);
						else if (angle>destangle && destangle-angle>=-180) this->Rotate(0);
						else if (angle>destangle && destangle-angle<-180) this->Rotate(1);
					}
					if (!(rand()%10) && !aifire) aifire=true;
					aiwait=SDL_GetTicks();
					break;
				default:
					break;
			}
		}
	}
	if (!strcmp(aiscript.c_str(),"HEATSEEK"))
	{
		dangle=atan2((sy-y),(sx-x));
		destangle=(int)(dangle*(180/PI))+90;
		if (destangle<0) destangle+=360;
		dstangle=destangle-(destangle%5);
		if (SDL_GetTicks()>=aiwait+aiwaitfor)
		{
			aiwaitfor=rand()%100;
			if (dstangle!=angle)
			{
				if (angle<destangle && destangle-angle>=180) this->Rotate(0);
				else if (angle<destangle && destangle-angle>=0) this->Rotate(1);
				else if (angle<destangle && destangle-angle<0) this->Rotate(0);
				else if (angle>destangle && destangle-angle>=-180) this->Rotate(0);
				else if (angle>destangle && destangle-angle<-180) this->Rotate(1);
			}
			aiwait=SDL_GetTicks();
		}
	}
	dx+=speed*(sin(angle*(PI/180)));
	dy+=speed*-(cos(angle*(PI/180)));
	if (speed>maxsafewarp)
	{
		if (SDL_GetTicks()>=overheatwait+2000 && overheatcount<overheatmax)
		{
			overheatcount++;
			overheatwait=SDL_GetTicks();
		}
	}
	else if (speed<=maxsafewarp)
	{
		if (SDL_GetTicks()>=overheatwait+2000 && overheatcount>0)
		{
			overheatcount--;
			overheatwait=SDL_GetTicks();
		}
	}
	x=(int)dx;
	y=(int)dy;
}
void Unit::SetMMR(Uint8 sr)
{
	mmr=sr;
}
void Unit::SetMMG(Uint8 sg)
{
	mmg=sg;
}
void Unit::SetMMB(Uint8 sb)
{
	mmb=sb;
}
void Unit::SetWarp(bool sw)
{
	warp=sw;
}
void Unit::SetName(std::string sname)
{
	name=sname;
}
void Unit::SetZoom(float szoom)
{
	zoom=szoom;
}
void Unit::SetMaxSpeed(int ms)
{
	maxspeed=ms;
}
void Unit::SetMaxWarp(int mw)
{
	maxwarp=mw;
}
void Unit::SetMaxSafeWarp(int msw)
{
	maxsafewarp=msw;
}
void Unit::SetTurnDelay(int td)
{
	turndelay=td;
}
void Unit::SetWarpTurnDelay(int tdw)
{
	warpturndelay=tdw;
}
void Unit::SetHullHitMax(int hhm)
{
	hullhitmax=hhm;
}
void Unit::SetShieldHitMax(int shm)
{
	if (hasShield && shield[0]) shieldhitmax=shm;
	else shieldhitmax=0;
}
void Unit::SetOverHeatMax(int ohm)
{
	overheatmax=ohm;
	overheatcount=0;
	overheatwait=SDL_GetTicks();
}
void Unit::SetShootPoint(int sx, int sy)
{
	origshootx=sx-(image[0]->w/2);
	origshooty=sy-(image[0]->h/2);
	this->AdjustShootPoint();
}
void Unit::SetCredits(unsigned int cred)
{
	credits=cred;
}
void Unit::AdjustShootPoint()
{
	double theta=(((angle)*(PI/180))+(atan2(origshooty,origshootx)))-(90*(PI/180));
	int r = (int)sqrt(origshootx*origshootx + origshooty*origshooty);
	shootx = (int)(r*-sin(theta));
	shooty = (int)(r*cos(theta));
}
void Unit::ResetHits()
{
	numhullhits=0;
	numshieldhits=0;
}
void Unit::SetHullHits(int hh)
{
	numhullhits=hh;
}
void Unit::SetShieldHits(int sh)
{
	numshieldhits=sh;
	if (numshieldhits<shieldhitmax) this->AddShield(shieldload);
}
void Unit::PrevWeapon()
{
	if (curweapon-1>=0) curweapon--;
	else if (weaponcount>0) curweapon=weaponcount-1;
	else curweapon=0;
}
void Unit::NextWeapon()
{
	if (curweapon+1<weaponcount) curweapon++;
	else curweapon=0;
}
void Unit::Rotate(int rotation)
{
	if (rotation==1)
	{
		if (angle+angleinc<360)
		{
			angle+=angleinc;
		}
		else
		{
			angle=0;
		}
		this->AdjustShootPoint();
	}
	else
	{
		if (angle-angleinc>=0)
		{
			angle-=angleinc;
		}
		else
		{
			angle=360-angleinc;
		}
		this->AdjustShootPoint();
	}
}
void Unit::EnableWarp()
{
	warpenabled=true;
}
void Unit::DisableWarp()
{
	warpenabled=false;
}
void Unit::AddShield(std::string shieldpic)
{
	int i;
	shieldload=shieldpic;
	hasShield=true;
	hadShield=true;
	SDL_Surface *temp;
	if (!strcmp(usingfpk.c_str()," ")) temp=LoadIMG(shieldpic);
	else temp=FPK_LoadIMG(usingfpk, shieldpic);
	SDL_Surface *temp2;
	for(i=0; i<UNITS_NUM_FRAMES; i++)
	{
		shield[i]=rotozoomSurface(temp, 360-i*angleinc, zoom, SMOOTH);
		temp2=rotozoomSurface(temp, 360-i*angleinc, zoom, SMOOTH);
		smask[i]=MakeMask(temp2);
		SDL_FreeSurface(temp2);
		temp2=0;
	}
	SDL_FreeSurface(temp);
	if (temp2) SDL_FreeSurface(temp2);
}
void Unit::AddWeaponToSlot(int slotid, std::string fpk, std::string fn, int amount)
{
	int i;
	for (i=0; i<numslots; i++)
	{
		if (slots[i].id==slotid && (!strcmp(slots[i].type.c_str(),"weapon") || !strcmp(slots[i].type.c_str(),"")))
		{
			slots[i].fpk=fpk;
			slots[i].fn=fn;
			if (!slots[i].ammoamount) weaponcount++;
			slots[i].fulltype="weapon";
			if (!slots[i].image) slots[i].image=FPK_LoadIMG(slots[i].fpk,slots[i].fn);
			slots[i].full=true;
			slots[i].weaponfull=true;
			if (amount) slots[i].ammoamount+=amount;
			if (slots[i].ammoamount>slots[i].holdamount) slots[i].ammoamount=slots[i].holdamount;
		}
	}
}
void Unit::RemoveWeaponFromSlot(int slotid)
{
	int i;
	for (i=0; i<numslots; i++)
	{
		if (slots[i].id==slotid && !strcmp(slots[i].fulltype.c_str(),"weapon"))
		{
			slots[i].fpk="";
			slots[i].fn="";
			slots[i].fulltype="";
			SDL_FreeSurface(slots[i].image);
			slots[i].ammoamount=0;
			slots[i].image=0;
			slots[i].weaponfull=false;
		}
	}
	if (weaponcount>0) weaponcount--;
}
void Unit::ReduceAmmo(int slotid)
{
	int i, j;
	i=0;
	for(j=0; j<numslots; j++)
	{
		if (slots[j].id==slotid) i=j;
	}
	if (slots[i].ammoamount) slots[i].ammoamount-=1;
	if (slots[i].ammoamount<=0 && slots[i].holdamount)
	{
		this->RemoveWeaponFromSlot(slotid);
		this->NextWeapon();
	}
}
void Unit::AddWeaponBayToSlot(int slotid, std::string holdweapon, int holdamount)
{
	int i;
	for (i=0; i<numslots; i++)
	{
		if (slots[i].id==slotid && (!strcmp(slots[i].type.c_str(),"weapon") || !strcmp(slots[i].type.c_str(),"")))
		{
			slots[i].full=true;
			slots[i].weaponfull=false;
			slots[i].fulltype="weapon";
			slots[i].holdweapon=holdweapon;
			slots[i].holdamount=holdamount;
			slots[i].ammoamount=0;
			slots[i].fpk="";
			slots[i].fn="";
		}
	}
}
void Unit::RemoveWeaponBayFromSlot(int slotid)
{
	int i;
	for (i=0; i<numslots; i++)
	{
		if (slots[i].id==slotid && !strcmp(slots[i].fulltype.c_str(),"weapon"))
		{
			slots[i].fpk="";
			slots[i].fn="";
			slots[i].fulltype="";
			if (slots[i].image) SDL_FreeSurface(slots[i].image);
			slots[i].image=0;
			slots[i].full=false;
			slots[i].weaponfull=false;
			slots[i].holdweapon="";
		}
	}
}
void Unit::AddCargoBayToSlot(int slotid, int units)
{
	int i;
	for (i=0; i<numslots; i++)
	{
		if (slots[i].id==slotid && (!strcmp(slots[i].type.c_str(),"cargo") || !strcmp(slots[i].type.c_str(),"")))
		{
			slots[i].full=true;
			slots[i].weaponfull=false;
			slots[i].fulltype="cargo";
			slots[i].fpk="";
			slots[i].fn="";
			slots[i].cargounits=units;
			cargounits+=units;
		}
	}
}
void Unit::RemoveCargoBayFromSlot(int slotid)
{
	int i;
	for (i=0; i<numslots; i++)
	{
		if (slots[i].id==slotid && !strcmp(slots[i].fulltype.c_str(),"cargo"))
		{
			slots[i].fpk="";
			slots[i].fn="";
			slots[i].fulltype="";
			slots[i].full=false;
			cargounits-=slots[i].cargounits;
			slots[i].cargounits=0;
		}
	}
}
void Unit::AddToCargo(__cargo newcargo)
{
	cargo[cargofill].type=newcargo.type;
	cargo[cargofill].priceBought=newcargo.priceBought;
	cargofill++;
	this->SortCargo();
}
void Unit::SortCargo()
{
	int j=1;
	while (j<cargofill)
	{
		__cargo key = cargo[j];
		int i = j - 1;
		while ((i >=0) && (cargo[i].type > key.type))
		{
			cargo[i+1] = cargo[i];
			i = i - 1;
		}
		cargo[i+1] = key;
		j+=1;
	}
	if (LOG)
	{
		for (j=0; j<cargofill; j++)
		{
			printf("cargo[%d].type=%s\n",j,cargo[j].type.c_str());
		}
	}
}
void Unit::RemoveFromCargo(int cargoindex)
{
	int i;
	for(i=cargoindex; i<cargofill-1; i++)
	{
		cargo[i].type=cargo[i+1].type;
		cargo[i].priceBought=cargo[i+1].priceBought;
	}
	cargofill--;
}
__ship_slot Unit::GetCurWeaponSlot()
{
	int i;
	int r=-1;
	int j=-1;
	for (i=0; i<numslots; i++)
	{
		if (!strcmp(slots[i].fulltype.c_str(),"weapon"))
		{
			j++;
		}
		if (j==curweapon && r==-1 && slots[j].weaponfull) r=i;
	}
	return slots[r];
}
int Unit::GetWidth()
{
	int ang;
	ang=(int)angle/angleinc;
	if (hasShield) return MAX(image[ang]->w,shield[ang]->w);
	else return image[ang]->w;
}
int Unit::GetHeight()
{
	int ang;
	ang=(int)angle/angleinc;
	if (hasShield) return MAX(image[ang]->h,shield[ang]->h);
	else return image[ang]->h;
}
int Unit::GetImageWidth()
{
	int ang;
	ang=(int)angle/angleinc;
	return image[ang]->w;
}
int Unit::GetImageHeight()
{
	int ang;
	ang=(int)angle/angleinc;
	return image[ang]->h;
}
int Unit::GetDispImageW()
{
	return dispimg->w;
}
int Unit::GetDispImageH()
{
	return dispimg->h;
}
Uint8 Unit::GetMMR()
{
	return mmr;
}
Uint8 Unit::GetMMG()
{
	return mmg;
}
Uint8 Unit::GetMMB()
{
	return mmb;
}
int Unit::GetSpeed()
{
	return speed;
}
int Unit::GetAngle()
{
	return angle;
}
int Unit::GetX()
{
	return x;
}
int Unit::GetY()
{
	return y;
}
double Unit::GetDX()
{
	return dx;
}
double Unit::GetDY()
{
	return dy;
}
int Unit::GetNegX()
{
	return -x;
}
int Unit::GetNegY()
{
	return -y;
}
int Unit::GetSectorX()
{
	return (int)x/1000;
}
int Unit::GetSectorY()
{
	return (int)y/1000;
}
int Unit::GetMaxSpeed()
{
	return maxspeed;
}
int Unit::GetMaxWarp()
{
	return maxwarp;
}
int Unit::GetMaxSafeWarp()
{
	return maxsafewarp;
}
int Unit::GetTurnDelay()
{
	return turndelay;
}
int Unit::GetWarpTurnDelay()
{
	return warpturndelay;
}
int Unit::GetMaxHullHits()
{
	return hullhitmax;
}
int Unit::GetHullHits()
{
	return numhullhits;
}
int Unit::GetMaxShieldHits()
{
	return shieldhitmax;
}
int Unit::GetShieldHits()
{
	return numshieldhits;
}
int Unit::GetShootX()
{
	return shootx+x;
}
int Unit::GetShootY()
{
	return shooty+y;
}
unsigned int Unit::GetCredits()
{
	return credits;
}
int Unit::GetCargoUnits()
{
	return cargounits;
}
int Unit::GetCargoFill()
{
	return cargofill;
}
float Unit::GetZoom()
{
	return zoom;
}
std::string Unit::GetName()
{
	return name;
}
std::string Unit::GetCurWeaponFN()
{
	return curweaponfn;
}
std::string Unit::GetCurWeaponFPK()
{
	return curweaponfpk;
}
bool Unit::AtWarp()
{
	return warp;
}
bool Unit::WarpEnabled()
{
	return warpenabled;
}
bool Unit::HasShield()
{
	return hasShield;
}
void Unit::Hit()
{
	int i;
	if (hasShield) numshieldhits+=1;
	else numhullhits+=1;
	if (numshieldhits>=shieldhitmax)
	{
		for (i=0; i<UNITS_NUM_FRAMES; i++)
		{
			if (hasShield && shield[i])
			{
				SDL_FreeSurface(shield[i]);
				shield[i]=0;
				bitmask_free(smask[i]);
			}
		}
		hasShield=false;
	}
}
bool Unit::IsAIFiring()
{
	bool temp=aifire;
	aifire=false;
	return temp;
}
bool Unit::IsOverHeated()
{
	if (overheatcount<overheatmax) return false;
	else return true;
}
void Unit::Orbit()
{
	double xloc, yloc, ts, factor;
	ts=(double)speed;
	factor=ts/radius;
	if (angle2+factor>=360) {angle2=angle2+factor-360;}
	else {angle2+=factor;}
	xloc = orbitx + radius*cos(angle2);
	yloc = orbity + radius*sin(angle2);
	x=(int)xloc;
	y=(int)yloc;
	if (angle+angleinc<360) { angle+=angleinc; }
	else { angle=0; }
}
void Unit::Orbit(int sx, int sy, int rad)
{
	angle2=rand()%360;
	double xloc, yloc, ts, factor;
	orbitx=sx;
	orbity=sy;
	radius=rad;
	ts=(double)speed;
	factor=ts/radius;
	if (angle2+factor>=360) {angle2=angle2+factor-360;}
	else {angle2+=factor;}
	xloc = orbitx + radius*cos(angle2);
	yloc = orbity + radius*sin(angle2);
	x=(int)xloc;
	y=(int)yloc;
}
void Unit::SetOrbit(bool orbit)
{
	orbiter=orbit;
}
bool Unit::IsOrbiter()
{
	return orbiter;
}
void Unit::FreeImages()
{
	int i;
	for (i=0; i<UNITS_NUM_FRAMES; i++)
	{
		if (image[i])
		{
			SDL_FreeSurface(image[i]);
			image[i]=0;
		}
		if (shield[i])
		{
			SDL_FreeSurface(shield[i]);
			shield[i]=0;
		}
		if (dispimg)
		{
			SDL_FreeSurface(dispimg);
			dispimg=0;
		}
	}
}
void Unit::FPK_NewImages(std::string fpk, std::string imagename)
{
	if (LOG) printf("FPK_NewImages: Loading new image (%s) from fpk %s\n",imagename.c_str(),fpk.c_str());
	usingfpk=fpk;
	imgload=imagename;
	if (LOG) printf("FPK_NewImages: Freeing existing images\n");
	FreeImages();
	if(hasShield)
	{
		int i;
		for(i=0; i<UNITS_NUM_FRAMES; i++)
		{
			bitmask_free(smask[i]);
		}
		hasShield=false;
	}
	if (LOG) printf("FPK_NewImages: Loading new images\n");
	LoadImages(0, 71);
	int i;
	if (LOG) printf("FPK_NewImages: Making new bitmasks\n");
	for (i=0; i<UNITS_NUM_FRAMES; i++)
	{
		bitmask_free(mask[i]);
		mask[i]=MakeMask(image[i]);
	}
}
void Unit::ReloadImages()
{
	LoadImages(0,71);
}
void Unit::LoadImages(int start, int end)
{
	int i;
	SDL_Surface *temp;
	if (end>71) { end=71; }
	if (start>71) { start=71; }
	if (end<0) { end=0; }
	if (start<0) { start=0; }
	if (LOG) printf("Unit::LoadImages: Loading base image (%s) from fpk %s\n",imgload.c_str(),usingfpk.c_str());
	temp=FPK_LoadIMG(usingfpk, imgload);
	if (dispimg)
	{
		SDL_FreeSurface(dispimg);
		dispimg=0;
	}
	if (LOG) printf("Unit::LoadImages: Setting display image\n");
	dispimg=rotozoomSurface(temp, 0, 0.5, SMOOTH);
	if (cr>0 || cg>0 || cb>0)
	{
		if (LOG) printf("Unit::LoadImages: Colorizing images\n");
		ColorizeSurface(temp, cr, cg, cb);
		ColorizeSurface(dispimg, cr, cg, cb);
	}
	if (LOG) printf("Unit::LoadImages: Freeing images and reloading: ");
	for (i=start; i<=end; i++)
	{
		if (LOG) printf("%d, ",i);
		if (image[i])
		{
			SDL_FreeSurface(image[i]);
			image[i]=0;
		}
		image[i]=rotozoomSurface(temp, 360-i*angleinc, zoom, SMOOTH);
	}
	if (LOG) printf("\n");
	if (hasShield)
	{
		if (LOG) printf("Loading shield\n");
		SDL_FreeSurface(temp);
		temp=0;
		if (!strcmp(usingfpk.c_str()," ")) temp=LoadIMG(shieldload);
		else temp=FPK_LoadIMG(usingfpk, shieldload);
		for (i=start; i<=end; i++)
		{
			if (shield[i])
			{
				SDL_FreeSurface(shield[i]);
				shield[i]=0;
			}
			shield[i]=rotozoomSurface(temp, 360-i*angleinc, zoom, SMOOTH);
		}
	}
	SDL_FreeSurface(temp);
}
bool Unit::IsLoaded(int pic)
{
  return bool(image[pic]!=0);
}
bool Unit::ShieldIsLoaded(int pic)
{
  return bool(shield[pic]!=0);
}
SDL_Surface *Unit::GetCurImage()
{
	return image[angle/5];
}
void Unit::DisplayDispImage(int x, int y)
{
	DrawIMG(dispimg,x,y);
}
void Unit::ZeroSlots()
{
	int i;
	for (i=0; i<__MAXSHIPSLOTS; i++)
	{
		slots[i].id=0;
		slots[i].w=0;
		slots[i].l=0;
		slots[i].d=0;
		slots[i].holdamount=0;
		slots[i].ammoamount=0;
		slots[i].cargounits=0;
		slots[i].type="";
		slots[i].full=false;
		slots[i].weaponfull=false;
		slots[i].fulltype="";
		slots[i].fpk="";
		slots[i].fn="";
		if (slots[i].image) SDL_FreeSurface(slots[i].image);
		slots[i].image=0;
	}
	numslots=0;
}
void Unit::FillSlots(__ship_slot *sls, int max)
{
	int i;
	for (i=0; i<max; i++)
	{
		slots[i].id=sls[i].id;
		slots[i].w=sls[i].w;
		slots[i].l=sls[i].l;
		slots[i].d=sls[i].d;
		slots[i].holdamount=sls[i].holdamount;
		slots[i].ammoamount=sls[i].ammoamount;
		slots[i].type=sls[i].type;
		slots[i].full=sls[i].full;
		slots[i].weaponfull=sls[i].weaponfull;
		slots[i].fulltype=sls[i].fulltype;
		slots[i].holdweapon=sls[i].holdweapon;
		slots[i].fpk=sls[i].fpk;
		slots[i].fn=sls[i].fn;
		if (slots[i].image) SDL_FreeSurface(slots[i].image);
		slots[i].image=sls[i].image;
	}
	numslots=max;
	curweapon=0;
	weaponcount=0;
}
__ship_slot *Unit::GetSlots()
{
	return slots;
}
__cargo *Unit::GetCargo()
{
	return cargo;
}
int Unit::GetNumSlots()
{
	return numslots;
}
int Unit::GetHeatSeekingBay()
{
	int id=0;
	bool got=false;
	int i;
	for (i=0; i<numslots; i++)
	{
		if (!strcmp(slots[i].holdweapon.c_str(),"heatseeking") && !got && slots[i].ammoamount<slots[i].holdamount)
		{
			id=slots[i].id;
			got=true;
		}
	}
	return id;
}
int Unit::GetHeatSeekingAmmoFill()
{
	int ammoamount=0;
	int i;
	for (i=0; i<numslots; i++)
	{
		if (!strcmp(slots[i].holdweapon.c_str(),"heatseeking"))
		{
			ammoamount+=slots[i].ammoamount;
		}
	}
	return ammoamount;
}
int Unit::GetHeatSeekingAmmoSpace()
{
	int holdamount=0;
	int i;
	for (i=0; i<numslots; i++)
	{
		if (!strcmp(slots[i].holdweapon.c_str(),"heatseeking"))
		{
			holdamount+=slots[i].holdamount;
		}
	}
	return holdamount;
}
void Unit::Colorize(Uint8 r, Uint8 g, Uint8 b)
{
	int i;
	cr=r;
	cg=g;
	cb=b;
	for(i=0; i<UNITS_NUM_FRAMES; i++)
	{
		if (image[i])
		{
			if (cr>0 || cg>0 || cb>0) ColorizeSurface(image[i], cr, cg, cb);
		}
	}
	if (dispimg)
	{
		if (cr>0 || cg>0 || cb>0) ColorizeSurface(dispimg, cr, cg, cb);
	}
}

MapObject::~MapObject()
{
	if (image) SDL_FreeSurface(image);
}
void MapObject::Init(int sx, int sy, std::string img)
{
	x=sx;
	y=sy;
	imgload=img;
	zoom=1;
	image=0;
	loaded=false;
	mmr=0;
	mmg=255;
	mmb=0;
}
void MapObject::Display(int sx, int sy)
{
	DrawIMG(image, sx, sy);
}
void MapObject::SetName(std::string n)
{
	name=n;
}
void MapObject::SetX(int sx)
{
	x=sx;
}
void MapObject::SetY(int sy)
{
	y=sy;
}
void MapObject::SetMMR(Uint8 sr)
{
	mmr=sr;
}
void MapObject::SetMMG(Uint8 sg)
{
	mmg=sg;
}
void MapObject::SetMMB(Uint8 sb)
{
	mmb=sb;
}
void MapObject::SetZoom(float szoom)
{
	zoom=szoom;
}
int MapObject::GetWidth()
{
	return image->w;
}
int MapObject::GetHeight()
{
	return image->h;
}
Uint8 MapObject::GetMMR()
{
	return mmr;
}
Uint8 MapObject::GetMMG()
{
	return mmg;
}
Uint8 MapObject::GetMMB()
{
	return mmb;
}
std::string MapObject::GetName()
{
	return name;
}
int MapObject::GetX()
{
	return x;
}
int MapObject::GetY()
{
	return y;
}
int MapObject::GetNegX()
{
	return -x;
}
int MapObject::GetNegY()
{
	return -y;
}
void MapObject::FreeImage()
{
	if (image) SDL_FreeSurface(image);
	image=0;
	loaded=false;
}
void MapObject::LoadImage()
{
	FreeImage();
	SDL_Surface *temp=LoadIMG(imgload);
	image=rotozoomSurface(temp, 0, zoom, SMOOTH);
	loaded=true;
	SDL_FreeSurface(temp);
}
int MapObject::GetSectorX()
{
	return (int)x/1000;
}
int MapObject::GetSectorY()
{
	return (int)y/1000;
}
bool MapObject::IsLoaded()
{
	return loaded;
}

Station::Station()
{
	int i;
	for (i=0; i<30; i++)
	{
		shop.buyitems[i].price=0;
		shop.buyitems[i].dispimage=0;
		shop.buyitems[i].bayw=0;
		shop.buyitems[i].bayl=0;
		shop.buyitems[i].bayd=0;
		shop.buyitems[i].amount=0;
		shop.buyitems[i].holdamount=0;
		shop.stock[i]=0;
		shop.sold[i]=0;
	}
	shop.buynext=0;
	cargonext=0;
	HRepairsAllowed=false;
	SRepairsAllowed=false;
	pm=2;
	ticks=SDL_GetTicks();
	buy=false;
	sell=false;
}
Station::~Station()
{
	int i;
	for (i=0; i<30; i++)
	{
		if (shop.buyitems[i].dispimage)
		{
			SDL_FreeSurface(shop.buyitems[i].dispimage);
			shop.buyitems[i].dispimage=0;
		}
	}
}
void Station::AddItemToBuyShop(__shopitem item)
{
	if (shop.buynext<30)
	{
		shop.buyitems[shop.buynext]=item;
		if (item.baseimport) shop.stock[shop.buynext]=item.baseimport;
		else shop.stock[shop.buynext]=-1;
		shop.sold[shop.buynext]=0;
		shop.buynext++;
	}
	buy=true;
}
void Station::AddSellableCargo(__cargo cargo)
{
	if(cargonext<30)
	{
		cargosell[cargonext].type=cargo.type;
		cargosell[cargonext].priceBought=cargo.priceBought;
		cargonext++;
	}
	sell=true;
}
void Station::AllowHullRepairs(unsigned int hullRepairCost)
{
	HRepairsAllowed=true;
	hrepairCost=hullRepairCost;
}
void Station::AllowShieldRepairs(unsigned int shieldRepairCost)
{
	SRepairsAllowed=true;
	srepairCost=shieldRepairCost;
}
void Station::Dock(Unit *ship)
{
	int done=0;
	SDL_Event event;
	SDL_Surface *screen=SDL_GetVideoSurface();
	int w=screen->w;
	int h=screen->h;
	SDL_Surface *bg = SDL_CreateRGBSurface(0, screen->w, screen->h, screen->format->BitsPerPixel, screen->format->Rmask, screen->format->Gmask, screen->format->Bmask, screen->format->Amask);
	SDL_Rect dest;
	dest.x = 0;
	dest.y = 0;
	SDL_BlitSurface(screen, NULL, bg, &dest);
	SDL_Color white = { 0xFF, 0xFF, 0xFF, 0 };
	TTF_Font *font=TTF_OpenFont("fonts/arial.ttf", 14);
	TTF_Font *font12=TTF_OpenFont("fonts/arial.ttf", 12);

	int choice=0;
	int buycur=0;
	int click, click2, getx, gety;
	int sPercentNeeded, hPercentNeeded;

	GUI *tabs=new GUI;
	tabs->Init();
	tabs->MakeWindow(60, 40, w-120, 400);
	int sBuy=tabs->MakeBigButton(80,90,60,60, "Buy",14);
	int sSell=tabs->MakeBigButton(80,152,60,60, "Sell",14);
	int sRepairs=tabs->MakeBigButton(80,214,60,60, "Repairs",14);
	int sLeaveStation=tabs->MakeButton((w/2)-50,410,100,"Leave Station");

	GUI *buyGUI=new GUI;
	buyGUI->Init();
	GUI *sellGUI=new GUI;
	sellGUI->Init();
	GUI *bayGUI=new GUI;
	bayGUI->Init();
	GUI *repairGUI=new GUI;
	repairGUI->Init();

	int sbPrev=buyGUI->MakeButton(180,380,50,"< Prev");
	int sbBuy=buyGUI->MakeButton(260,380,90,"Buy this");
	int sbNext=buyGUI->MakeButton(380,380,50,"Next >");
	int sbAllTypes=buyGUI->MakeButton(160,100,70,"All");

	int sbayPrev=bayGUI->MakeButton(180,380,50,"< Prev");
	int sbaySelect=bayGUI->MakeButton(260,380,90,"Select");
	int sbayNext=bayGUI->MakeButton(380,380,50,"Next >");
	int sbayCancel=bayGUI->MakeButton(460,380,50,"Cancel");

	int ssPrev=sellGUI->MakeButton(180,380,50,"< Prev");
	int ssSell=sellGUI->MakeButton(260,380,90,"Sell");
	int ssNext=sellGUI->MakeButton(380,380,50,"Next >");
	int ssCargo=sellGUI->MakeButton(160,100,70,"Cargo");


	int rHull=-1;
	int rShield=-1;
	if (HRepairsAllowed && SRepairsAllowed)
	{
		rHull=repairGUI->MakeButton(w/2-95,h/2,90, "Repair Hull");
		rShield=repairGUI->MakeButton(w/2+5,h/2,90, "Recharge Shields");
	}
	else
	{
		if (HRepairsAllowed) rHull=repairGUI->MakeButton(w/2-45,h/2,90, "Repair Hull");
		if (SRepairsAllowed) rShield=repairGUI->MakeButton(w/2-45,h/2,90, "Recharge Shields");
	}

	hPercentNeeded=(int)(((float)ship->GetHullHits()/(float)ship->GetMaxHullHits())*100);
	sPercentNeeded=(int)(((float)ship->GetShieldHits()/(float)ship->GetMaxShieldHits())*100);

	__ship_slot *slots;
	slots=ship->GetSlots();
	int numslots=ship->GetNumSlots();
	int slotcur=0;

	__cargo *cargohold;
	cargohold=ship->GetCargo();
	int cargofill=ship->GetCargoFill();
	std::string cargotypecur=cargohold[0].type;
	int cargocur=0;
	int curcargocount=0;
	int cargosellcur=0;

	int buying=0;

	char disp[300];

	while(!done)
	{
		Slock();
		while (SDL_PollEvent(&event))
		{
			switch (event.type) {
			// close button clicked
			case SDL_QUIT:
				exit(0);
				break;

			// handle the keyboard
			case SDL_KEYDOWN:
				switch (event.key.keysym.sym) {
				case SDLK_ESCAPE:
					done = 1;
					break;
				default:
					break;
				}
				break;
			case SDL_MOUSEMOTION:
				SDL_GetMouseState(&getx, &gety);
				tabs->SendMouseMotion(getx,gety);
				if (choice==0 && buying==0) buyGUI->SendMouseMotion(getx, gety);
				if (choice==0 && buying==1) bayGUI->SendMouseMotion(getx, gety);
				if (choice==1) sellGUI->SendMouseMotion(getx, gety);
				if (choice==2) repairGUI->SendMouseMotion(getx, gety);
				break;
			case SDL_MOUSEBUTTONDOWN:
				SDL_GetMouseState(&getx, &gety);
				click = tabs->SendMouseClick(getx, gety);
				if (choice==0 && buying==0) click2 = buyGUI->SendMouseClick(getx, gety);
				if (choice==0 && buying==1) click2 = bayGUI->SendMouseClick(getx, gety);
				if (choice==1) click2 = sellGUI->SendMouseClick(getx, gety);
				if (choice==2) click2 = repairGUI->SendMouseClick(getx, gety);
				break;
			case SDL_MOUSEBUTTONUP:
				SDL_GetMouseState(&getx, &gety);
				if (click!=-1 && click==tabs->SendMouseClick(getx, gety))
				{
					DrawIMG(bg,0,0);
					tabs->Draw();
					if (choice==2) repairGUI->Draw();
					centerString(font,w/2,54,this->GetName(),white);
					sprintf(disp, "Credits: %d",ship->GetCredits());
					drawString(font12, w-160, 70, disp, white);
					Sulock();
					SDL_Flip(screen);
					if (click==sBuy)
					{
						choice=0;
						buying=0;
					}
					if (click==sSell)
					{
						choice=1;
						if (sell)
						{
							cargofill=ship->GetCargoFill();
							cargotypecur=cargohold[0].type;
							cargocur=0;
							curcargocount=0;
							int i;
							for(i=0; i<cargofill; i++)
							{
								if(!strcmp(cargotypecur.c_str(),cargohold[i].type.c_str())) curcargocount++;
							}
							for(i=0;i<cargonext;i++)
							{
								if(!strcmp(cargotypecur.c_str(),cargosell[i].type.c_str())) cargosellcur=i;
							}
						}
					}
					if (click==sRepairs)
					{
						choice=2;
					}
					if (click==sLeaveStation)
					{
						done=1;
					}
				}
				if (click2!=-1 && (click2==buyGUI->SendMouseClick(getx, gety) || click2==sellGUI->SendMouseClick(getx, gety) || click2==bayGUI->SendMouseClick(getx, gety) || click2==repairGUI->SendMouseClick(getx, gety)))
				{
					DrawIMG(bg,0,0);
					tabs->Draw();
					if (choice==0 && buying==0)
					{
						if (buy)
						{
							if (shop.buyitems[buycur].dispimage) DrawIMG(shop.buyitems[buycur].dispimage,150,140);
							if (!strcmp(shop.buyitems[buycur].type.c_str(),"cargo"))
							{
								sprintf(disp, "Cargo Space: %d/%d",cargofill,ship->GetCargoUnits());
								drawString(font12, 150, 340, disp, white);
							}
							if (!strcmp(shop.buyitems[buycur].type.c_str(),"ammo"))
							{
								sprintf(disp, "Ammo Space: %d/%d",ship->GetHeatSeekingAmmoFill(),ship->GetHeatSeekingAmmoSpace());
								drawString(font12, 150, 340, disp, white);
							}
							sprintf(disp, "Name: %s",shop.buyitems[buycur].name.c_str());
							drawString(font12, 260, 140, disp, white);
							sprintf(disp, "Price: %d credits",shop.buyitems[buycur].price);
							drawString(font12, 260, 160, disp, white);
							sprintf(disp, "Description: %s",shop.buyitems[buycur].desc.c_str());
							drawString(font12, 260, 180, disp, white);
							if (shop.stock[buycur]>=0)
							{
								sprintf(disp, "Stock: %d",shop.stock[buycur]);
								drawString(font12, 260, 200, disp, white);
							}
							buyGUI->Draw();
							if (click2==sbPrev)
							{
								if (buycur-1<0) buycur=shop.buynext-1;
								else buycur-=1;
							}
							else if (click2==sbNext)
							{
								if (buycur+1>=shop.buynext) buycur=0;
								else buycur+=1;
							}
							else if (click2==sbBuy)
							{
								if (!strcmp("bay",shop.buyitems[buycur].type.c_str()))
								{
									if (ship->GetCredits()>=(int)shop.buyitems[buycur].price)
									{
										slotcur=0;
										buying=1;
									}
								}
								else if (!strcmp("cargo",shop.buyitems[buycur].type.c_str()))
								{
									if (shop.stock[buycur]>0 && ship->GetCredits()>=(int)shop.buyitems[buycur].price && ship->GetCargoFill()<ship->GetCargoUnits())
									{
										__cargo newcargo;
										newcargo.type=shop.buyitems[buycur].name;
										newcargo.priceBought=shop.buyitems[buycur].price;
										ship->AddToCargo(newcargo);
										ship->SetCredits(ship->GetCredits()-shop.buyitems[buycur].price);
										shop.sold[buycur]+=1;
										shop.stock[buycur]-=1;
									}
									//else the cargo bays are full
								}
								else if (!strcmp("ammo",shop.buyitems[buycur].type.c_str()))
								{
									int bayid=ship->GetHeatSeekingBay();
									if (ship->GetCredits()>=(int)shop.buyitems[buycur].price && bayid)
									{
										ship->SetCredits(ship->GetCredits()-shop.buyitems[buycur].price);
										ship->AddWeaponToSlot(bayid,shop.buyitems[buycur].fpk,shop.buyitems[buycur].imagefn,shop.buyitems[buycur].amount);
										if (LOG) printf("Inserted %d of ammo into bay id %d\n",shop.buyitems[buycur].amount,bayid);
									}
								}
								else
								{
									if (shop.stock[buycur]>0 && ship->GetCredits()>=(int)shop.buyitems[buycur].price)
									{
										ship->SetCredits(ship->GetCredits()-shop.buyitems[buycur].price);
										shop.sold[buycur]+=1;
										shop.stock[buycur]-=1;
									}
									if (shop.stock[buycur]==-1 && ship->GetCredits()>=(int)shop.buyitems[buycur].price)
									{
										ship->SetCredits(ship->GetCredits()-shop.buyitems[buycur].price);
										//give it
									}
								}
							}
						}
						else centerString(font,w/2,h/2,"No Buying at this station",white);
					}
					else if (choice==0 && buying==1)
					{
						sprintf(disp, "Selected Bay: %s",shop.buyitems[buycur].name.c_str());
						drawString(font12, 160, 90, disp, white);
						sprintf(disp, "Bay Width: %d  Bay Length: %d  Bay Depth: %d",shop.buyitems[buycur].bayw,shop.buyitems[buycur].bayl,shop.buyitems[buycur].bayd);
						drawString(font12, 160, 110, disp, white);
						if (!slots[slotcur].full)
						{
							if (!strcmp(slots[slotcur].type.c_str(),"") ||
							(!strcmp(slots[slotcur].type.c_str(),shop.buyitems[buycur].baytype.c_str())))
							{
								sprintf(disp, "Place in your ship slot %d?",slots[slotcur].id);
								drawString(font12, 160, 140, disp, white);
								sprintf(disp, "Bay Width: %d  Bay Length: %d  Bay Depth: %d",slots[slotcur].w,slots[slotcur].l,slots[slotcur].d);
								drawString(font12, 160, 160, disp, white);
							}
							else
							{
								sprintf(disp, "Ship slot %d does not take %s bays",slots[slotcur].id,shop.buyitems[buycur].baytype.c_str());
								drawString(font12, 160, 140, disp, white);
							}
						}
						else
						{
							sprintf(disp, "Ship slot %d is full",slots[slotcur].id);
							drawString(font12, 160, 140, disp, white);
						}
						bayGUI->Draw();
						if (click2==sbayPrev)
						{
							if (slotcur-1>=0) slotcur--;
							else slotcur=numslots-1;
						}
						else if (click2==sbayNext)
						{
							if (slotcur+1<numslots) slotcur++;
							else slotcur=0;
						}
						else if (click2==sbaySelect)
						{
							if (!slots[slotcur].full && slots[slotcur].w>=shop.buyitems[buycur].bayw && slots[slotcur].d>=shop.buyitems[buycur].bayd && slots[slotcur].l>=shop.buyitems[buycur].bayl)
							{
								if ((!strcmp(slots[slotcur].type.c_str(),"") || !strcmp(slots[slotcur].type.c_str(),"weapon")) && !strcmp(shop.buyitems[buycur].baytype.c_str(),"weapon"))
								{
									ship->AddWeaponBayToSlot(slots[slotcur].id, shop.buyitems[buycur].holdweapon, shop.buyitems[buycur].holdamount);
									ship->SetCredits(ship->GetCredits()-shop.buyitems[buycur].price);
								}
								if ((!strcmp(slots[slotcur].type.c_str(),"") || !strcmp(slots[slotcur].type.c_str(),"cargo")) && !strcmp(shop.buyitems[buycur].baytype.c_str(),"cargo"))
								{
									ship->AddCargoBayToSlot(slots[slotcur].id,shop.buyitems[buycur].cargounits);
									ship->SetCredits(ship->GetCredits()-shop.buyitems[buycur].price);
								}
							}
						}
						else if (click2==sbayCancel)
						{
							buying=0;
						}
					}
					else if (choice==1)
					{
						if (sell)
						{
							if (cargofill)
							{
								sprintf(disp, "Sell Cargo: %s",cargotypecur.c_str());
								drawString(font12, 160, 140, disp, white);
								sprintf(disp, "In Cargo Hold: %d",curcargocount);
								drawString(font12, 160, 160, disp, white);
								sprintf(disp, "Price: %d",cargosell[cargosellcur].priceBought);
								drawString(font12, 160, 180, disp, white);
							}
							else
							{
								drawString(font12, 160, 140, "No Cargo to Sell", white);
							}
							sprintf(disp, "Cargo Space: %d/%d",cargofill,ship->GetCargoUnits());
							drawString(font12, 150, 340, disp, white);
							sellGUI->Draw();
							if (click2==ssPrev)
							{
								int i=0;
								bool cycled=false;
								for(i=cargocur; i>=-1; i--)
								{
									if(strcmp(cargotypecur.c_str(),cargohold[i].type.c_str()))
									{
										cargocur=i;
										cargotypecur=cargohold[i].type;
										break;
									}
									if(i==0)
									{
										i=cargofill-1;
										cycled=true;
									}
									if(cycled && i==cargocur) break;
								}
								curcargocount=0;
								for(i=0; i<cargofill; i++)
								{
									if(!strcmp(cargotypecur.c_str(),cargohold[i].type.c_str())) curcargocount++;
								}
								for(i=0;i<cargonext;i++)
								{
									if(!strcmp(cargotypecur.c_str(),cargosell[i].type.c_str())) cargosellcur=i;
								}
							}
							else if (click2==ssNext)
							{
								int i=0;
								bool cycled=false;
								for(i=cargocur; i<cargofill+1; i++)
								{
									if(strcmp(cargotypecur.c_str(),cargohold[i].type.c_str()))
									{
										cargocur=i;
										cargotypecur=cargohold[i].type;
										break;
									}
									if(i==cargofill-1)
									{
										i=0;
										cycled=true;
									}
									if(cycled && i==cargocur) break;
								}
								curcargocount=0;
								for(i=0; i<cargofill; i++)
								{
									if(!strcmp(cargotypecur.c_str(),cargohold[i].type.c_str())) curcargocount++;
								}
								for(i=0;i<cargonext;i++)
								{
									if(!strcmp(cargotypecur.c_str(),cargosell[i].type.c_str())) cargosellcur=i;
								}
							}
							else if (click2==ssSell && cargofill)
							{
								ship->SetCredits(ship->GetCredits()+cargosell[cargocur].priceBought);
								ship->RemoveFromCargo(cargocur);
								cargofill=ship->GetCargoFill();
								if (cargocur+1<cargofill) cargocur++;
								else cargocur=0;
								curcargocount--;
								cargotypecur=cargohold[cargocur].type;
								int i;
								if (cargofill && curcargocount<=0)
								{
									curcargocount=0;
									for(i=0; i<cargofill; i++)
									{
										if(!strcmp(cargotypecur.c_str(),cargohold[i].type.c_str())) curcargocount++;
									}
								}
								for(i=0;i<cargonext;i++)
								{
									if(!strcmp(cargotypecur.c_str(),cargosell[i].type.c_str())) cargosellcur=i;
								}
							}
						}
						else centerString(font,w/2,h/2,"No Selling at this station",white);
					}
					else if (choice==2)
					{
						repairGUI->Draw();
						int inc=100;
						if (HRepairsAllowed)
						{
							sprintf(disp, "Hull Damage: %d%%",hPercentNeeded);
							drawString(font12, 160, inc, disp, white);
							inc+=16;
							sprintf(disp, "Price to Fully Repair Hull: %d",hPercentNeeded*hrepairCost);
							drawString(font12, 160, inc, disp, white);
							inc+=16;
						}
						if (SRepairsAllowed)
						{
							sprintf(disp, "Shield Damage: %d%%",sPercentNeeded);
							drawString(font12, 160, inc, disp, white);
							inc+=16;
							sprintf(disp, "Price to Fully Recharge Shields: %d",sPercentNeeded*srepairCost);
							drawString(font12, 160, inc, disp, white);
							inc+=16;
						}
						if (HRepairsAllowed || SRepairsAllowed)
						{
							inc+=4;
							drawString(font12, 160, inc, "Partial Repairs can be made", white);
						}
						else drawString(font12, 160, inc, "No Repairs can be made here", white);
					}
					centerString(font,w/2,54,this->GetName(),white);
					sprintf(disp, "Credits: %d",ship->GetCredits());
					drawString(font12, w-160, 70, disp, white);
					Sulock();
					SDL_Flip(screen);
					if (click2==rHull && choice==2)
					{
						float increment=((float)1/(float)ship->GetMaxHullHits())*100;
						if (ship->GetCredits()<(hPercentNeeded*hrepairCost))
						{
							float take=0;
							while ((take+increment)*hrepairCost<=(float)ship->GetCredits())
							{
								take+=increment;
							}
							ship->SetCredits(ship->GetCredits()-(((int)take)*hrepairCost));
							ship->SetHullHits((int)(ship->GetHullHits()-((float)((int)take)/(float)hrepairCost)));
						}
						else
						{
							ship->SetCredits(ship->GetCredits()-(hPercentNeeded*hrepairCost));
							ship->SetHullHits(0);
						}
						hPercentNeeded=(int)(((float)ship->GetHullHits()/(float)ship->GetMaxHullHits())*100);
					}
					if (click2==rShield && choice==2)
					{
						float increment=((float)1/(float)ship->GetMaxShieldHits())*100;
						if (ship->GetCredits()<(sPercentNeeded*srepairCost))
						{
							float take=0;
							while ((take+increment)*srepairCost<=(float)ship->GetCredits())
							{
								take+=increment;
							}
							ship->SetCredits(ship->GetCredits()-(((int)take)*srepairCost));
							ship->SetShieldHits((int)(ship->GetShieldHits()-((float)((int)take)/(float)srepairCost)));
						}
						else
						{
							ship->SetCredits(ship->GetCredits()-(sPercentNeeded*srepairCost));
							ship->SetShieldHits(0);
						}
						sPercentNeeded=(int)(((float)ship->GetShieldHits()/(float)ship->GetMaxShieldHits())*100);
					}
					cargofill=ship->GetCargoFill();
				}
				tabs->UpdateButtons();
				if (choice==0 && buying==0) buyGUI->UpdateButtons();
				else if (choice==0 && buying==1) bayGUI->UpdateButtons();
				else if (choice==1) sellGUI->UpdateButtons();
				else if (choice==2) repairGUI->UpdateButtons();
				break;
			default:
				break;
			}
		}
		DrawIMG(bg,0,0);
		tabs->Draw();
		centerString(font,w/2,54,this->GetName(),white);
		sprintf(disp, "Credits: %d",ship->GetCredits());
		drawString(font12, w-160, 70, disp, white);
		switch (choice)
		{
			int inc;
			case 0:
				if (buy)
				{
					if (buying==0)
					{
						if (shop.buyitems[buycur].dispimage) DrawIMG(shop.buyitems[buycur].dispimage,150,140);
						if (!strcmp(shop.buyitems[buycur].type.c_str(),"cargo"))
						{
							sprintf(disp, "Cargo Space: %d/%d",cargofill,ship->GetCargoUnits());
							drawString(font12, 150, 340, disp, white);
						}
						if (!strcmp(shop.buyitems[buycur].type.c_str(),"ammo"))
						{
							sprintf(disp, "Ammo Space: %d/%d",ship->GetHeatSeekingAmmoFill(),ship->GetHeatSeekingAmmoSpace());
							drawString(font12, 150, 340, disp, white);
						}
						sprintf(disp, "Name: %s",shop.buyitems[buycur].name.c_str());
						drawString(font12, 260, 140, disp, white);
						sprintf(disp, "Price: %d credits",shop.buyitems[buycur].price);
						drawString(font12, 260, 160, disp, white);
						sprintf(disp, "Description: %s",shop.buyitems[buycur].desc.c_str());
						drawString(font12, 260, 180, disp, white);
						if (shop.stock[buycur]>=0)
						{
							sprintf(disp, "Stock: %d",shop.stock[buycur]);
							drawString(font12, 260, 200, disp, white);
						}
						buyGUI->Draw();
					}
					if (buying==1)
					{
						sprintf(disp, "Selected Bay: %s",shop.buyitems[buycur].name.c_str());
						drawString(font12, 160, 90, disp, white);
						sprintf(disp, "Bay Width: %d  Bay Length: %d  Bay Depth: %d",shop.buyitems[buycur].bayw,shop.buyitems[buycur].bayl,shop.buyitems[buycur].bayd);
						drawString(font12, 160, 110, disp, white);
						if (!slots[slotcur].full)
						{
							if (!strcmp(slots[slotcur].type.c_str(),"") ||
							(!strcmp(slots[slotcur].type.c_str(),shop.buyitems[buycur].baytype.c_str())))
							{
								sprintf(disp, "Place in your ship slot %d?",slots[slotcur].id);
								drawString(font12, 160, 140, disp, white);
								sprintf(disp, "Bay Width: %d  Bay Length: %d  Bay Depth: %d",slots[slotcur].w,slots[slotcur].l,slots[slotcur].d);
								drawString(font12, 160, 160, disp, white);
							}
							else
							{
								sprintf(disp, "Ship slot %d does not take %s bays",slots[slotcur].id,shop.buyitems[buycur].baytype.c_str());
								drawString(font12, 160, 140, disp, white);
							}
						}
						else
						{
							sprintf(disp, "Ship slot %d is full",slots[slotcur].id);
							drawString(font12, 160, 140, disp, white);
						}
						bayGUI->Draw();
					}
				}
				else centerString(font,w/2,h/2,"No Buying at this station",white);
				break;
			case 1:
				if (sell)
				{
					if (cargofill)
					{
						sprintf(disp, "Sell Cargo: %s",cargotypecur.c_str());
						drawString(font12, 160, 140, disp, white);
						sprintf(disp, "In Cargo Hold: %d",curcargocount);
						drawString(font12, 160, 160, disp, white);
						sprintf(disp, "Price: %d",cargosell[cargosellcur].priceBought);
						drawString(font12, 160, 180, disp, white);
					}
					else
					{
						drawString(font12, 160, 140, "No Cargo to Sell", white);
					}
					sprintf(disp, "Cargo Space: %d/%d",cargofill,ship->GetCargoUnits());
					drawString(font12, 150, 340, disp, white);
					sellGUI->Draw();
				}
				else centerString(font,w/2,h/2,"No Selling at this station",white);
				break;
			case 2:
				inc=100;
				if (HRepairsAllowed)
				{
					sprintf(disp, "Hull Damage: %d%%",hPercentNeeded);
					drawString(font12, 160, inc, disp, white);
					inc+=16;
					sprintf(disp, "Price to Fully Repair Hull: %d",hPercentNeeded*hrepairCost);
					drawString(font12, 160, inc, disp, white);
					inc+=16;
				}
				if (SRepairsAllowed)
				{
					sprintf(disp, "Shield Damage: %d%%",sPercentNeeded);
					drawString(font12, 160, inc, disp, white);
					inc+=16;
					sprintf(disp, "Price to Fully Recharge Shields: %d",sPercentNeeded*srepairCost);
					drawString(font12, 160, inc, disp, white);
					inc+=16;
				}
				if (HRepairsAllowed || SRepairsAllowed)
				{
					inc+=4;
					drawString(font12, 160, inc, "Partial Repairs can be made", white);
				}
				else drawString(font12, 80, inc, "No Repairs can be made here", white);
				repairGUI->Draw();
				break;
			default:
				break;
		}
		Sulock();
		SDL_Flip(screen);
	}
	TTF_CloseFont(font);
	TTF_CloseFont(font12);
	delete tabs;
	delete buyGUI;
	delete sellGUI;
	delete bayGUI;
	delete repairGUI;
	ticks=SDL_GetTicks();
}
void Station::ChangeSupply()
{
	int i;
	for(i=0; i<30; i++)
	{
		if (shop.buyitems[i].baseprice)
		{
			shop.stock[i]+=shop.buyitems[i].baseimport;
			unsigned int temp=shop.buyitems[i].baseprice-pm;
			temp-=pm*(int)((double)shop.buyitems[i].baseimport/(double)shop.sold[i]);
			shop.sold[i]=0;
		}
	}
	ticks=SDL_GetTicks();
}
Uint32 Station::GetTicks()
{
	return ticks;
}

Map::~Map()
{
	int i;
	for (i=0; i<next; i++)
	{
		delete objects[i];
	}
	for (i=0; i<stationnext; i++)
	{
		delete stations[i];
	}
	for (i=0; i<actnext; i++)
	{
		delete actobjects[i];
	}
	for (i=0; i<snext; i++)
	{
		delete sobjects[i];
	}
	for (i=0; i<aisnext; i++)
	{
		delete aisobjects[i];
	}
	for (i=0; i<animnext; i++)
	{
		delete anims[i];
	}
	for (i=0; i<aiunitnext; i++)
	{
		delete aiunits[i];
	}
}
void Map::Init()
{
	int i;
	next=0;
	stationnext=0;
	actnext=0;
	snext=0;
	aisnext=0;
	animnext=0;
	activeanimnext=0;
	aiunitnext=0;
	zoom=1;
	for (i=0; i<20; i++)
	{
		activeanim[i]=false;
	}
}
void Map::AddMainShip(Unit *object)
{
	mainship=object;
}
void Map::AddToObjects(MapObject *object)
{
	//while (next>=10000) this->RemoveFromObjects(0);
	objects[next]=object;
	next+=1;
}
void Map::AddToStations(Station *object)
{
	//while (next>=10000) this->RemoveFromObjects(0);
	stations[stationnext]=object;
	stationnext+=1;
}
void Map::AddToActiveObjects(Unit *object)
{
	while (actnext>=1000) this->RemoveFromActiveObjects(0);
	actobjects[actnext]=object;
	actnext+=1;
}
void Map::RemoveFromActiveObjects(int start)
{
	int i;
	actobjects[start]->FreeImages();
	delete actobjects[start];
	for (i=start; i<actnext-1; i++)
	{
		actobjects[i]=actobjects[i+1];
	}
	actnext-=1;
}
void Map::AddToShootingObjects(Unit *object, int aishipindex)
{
	while (snext>=100) this->RemoveFromShootingObjects(0);
	sobjects[snext]=object;
	s_ticks[snext]=SDL_GetTicks();
	shootindex[snext]=aishipindex;
	snext+=1;
}
void Map::RemoveFromShootingObjects(int j)
{
	int i;
	sobjects[j]->FreeImages();
	delete sobjects[j];
	for (i=j; i<snext-1; i++)
	{
		sobjects[i]=sobjects[i+1];
		s_ticks[i]=s_ticks[i+1];
	}
	snext-=1;
}
void Map::AddToAIShootingObjects(Unit *object)
{
	while (aisnext>=100) this->RemoveFromAIShootingObjects(0);
	aisobjects[aisnext]=object;
	ais_ticks[aisnext]=SDL_GetTicks();
	aisnext+=1;
}
void Map::RemoveFromAIShootingObjects(int j)
{
	int i;
	aisobjects[j]->FreeImages();
	delete aisobjects[j];
	for (i=j; i<aisnext-1; i++)
	{
		aisobjects[i]=aisobjects[i+1];
		ais_ticks[i]=ais_ticks[i+1];
	}
	aisnext-=1;
}
void Map::DestroyObjects()
{
	int i, j;
	j=next;
	for (i=0; i<j; i++)
	{
		objects[i]->FreeImage();
		delete objects[i];
	}
	next=0;
	j=stationnext;
	for (i=0; i<j; i++)
	{
		stations[i]->FreeImage();
		delete stations[i];
	}
	stationnext=0;
	j=actnext;
	for (i=0; i<j; i++)
	{
		actobjects[i]->FreeImages();
		delete actobjects[i];
	}
	actnext=0;
	j=snext;
	for (i=0; i<j; i++)
	{
		sobjects[i]->FreeImages();
		delete sobjects[i];
	}
	snext=0;
	j=aisnext;
	for (i=0; i<j; i++)
	{
		aisobjects[i]->FreeImages();
		delete aisobjects[i];
	}
	aisnext=0;
	j=animnext;
	for (i=0; i<j; i++)
	{
		delete anims[i];
	}
	animnext=0;
	j=aiunitnext;
	for (i=0; i<j; i++)
	{
		delete aiunits[i];
	}
	aiunitnext=0;
}
void Map::DisplayMinimap(int x, int y, int shipx, int shipy, int range)
{
	SDL_Surface *screen=SDL_GetVideoSurface();
	int i, centx, centy, tx, ty;
	Uint8 r, g, b;
	centx=x+30;
	centy=y+30;
	boxRGBA(screen, x, y, x+60, y+60, 128,128,128,164);
	for (i=0; i<next; i++)
	{
		if (objects[i]->GetX() >= shipx-(range*1000) && objects[i]->GetX() <= shipx+(range*1000) && objects[i]->GetY() >= shipy-(range*1000) && objects[i]->GetY() <= shipy+(range*1000))
		{
			r=objects[i]->GetMMR();
			g=objects[i]->GetMMG();
			b=objects[i]->GetMMB();
			tx=(int)(((objects[i]->GetX()-shipx)/((range*1000)/30))+30+x);
			ty=(int)(((objects[i]->GetY()-shipy)/((range*1000)/30))+30+y);
			boxRGBA(screen, tx-1, ty-1, tx+1, ty+1, r,g,b,255);
		}
	}
	for (i=0; i<stationnext; i++)
	{
		if (stations[i]->GetX() >= shipx-(range*1000) && stations[i]->GetX() <= shipx+(range*1000) && stations[i]->GetY() >= shipy-(range*1000) && stations[i]->GetY() <= shipy+(range*1000))
		{
			r=stations[i]->GetMMR();
			g=stations[i]->GetMMG();
			b=stations[i]->GetMMB();
			tx=(int)(((stations[i]->GetX()-shipx)/((range*1000)/30))+30+x);
			ty=(int)(((stations[i]->GetY()-shipy)/((range*1000)/30))+30+y);
			boxRGBA(screen, tx-1, ty-1, tx+1, ty+1, r,g,b,255);
		}
	}
	for (i=0; i<actnext; i++)
	{
		if (actobjects[i]->GetX() >= shipx-(range*1000) && actobjects[i]->GetX() <= shipx+(range*1000) && actobjects[i]->GetY() >= shipy-(range*1000) && actobjects[i]->GetY() <= shipy+(range*1000))
		{
			r=actobjects[i]->GetMMR();
			g=actobjects[i]->GetMMG();
			b=actobjects[i]->GetMMB();
			tx=(int)(((actobjects[i]->GetX()-shipx)/((range*1000)/30))+30+x);
			ty=(int)(((actobjects[i]->GetY()-shipy)/((range*1000)/30))+30+y);
			boxRGBA(screen, tx-1, ty-1, tx+1, ty+1, r,g,b,255);
		}
	}
	for (i=0; i<snext; i++)
	{
		if (sobjects[i]->GetX() >= shipx-(range*1000) && sobjects[i]->GetX() <= shipx+(range*1000) && sobjects[i]->GetY() >= shipy-(range*1000) && sobjects[i]->GetY() <= shipy+(range*1000))
		{
			r=sobjects[i]->GetMMR();
			g=sobjects[i]->GetMMG();
			b=sobjects[i]->GetMMB();
			tx=(int)(((sobjects[i]->GetX()-shipx)/((range*1000)/30))+30+x);
			ty=(int)(((sobjects[i]->GetY()-shipy)/((range*1000)/30))+30+y);
			boxRGBA(screen, tx-1, ty-1, tx+1, ty+1, r,g,b,255);
		}
	}
	for (i=0; i<aisnext; i++)
	{
		if (aisobjects[i]->GetX() >= shipx-(range*1000) && aisobjects[i]->GetX() <= shipx+(range*1000) && aisobjects[i]->GetY() >= shipy-(range*1000) && aisobjects[i]->GetY() <= shipy+(range*1000))
		{
			r=aisobjects[i]->GetMMR();
			g=aisobjects[i]->GetMMG();
			b=aisobjects[i]->GetMMB();
			tx=(int)(((aisobjects[i]->GetX()-shipx)/((range*1000)/30))+30+x);
			ty=(int)(((aisobjects[i]->GetY()-shipy)/((range*1000)/30))+30+y);
			boxRGBA(screen, tx-1, ty-1, tx+1, ty+1, r,g,b,255);
		}
	}
	for (i=0; i<aiunitnext; i++)
	{
		if (aiunits[i]->GetX() >= shipx-(range*1000) && aiunits[i]->GetX() <= shipx+(range*1000) && aiunits[i]->GetY() >= shipy-(range*1000) && aiunits[i]->GetY() <= shipy+(range*1000))
		{
			r=aiunits[i]->GetMMR();
			g=aiunits[i]->GetMMG();
			b=aiunits[i]->GetMMB();
			tx=(int)(((aiunits[i]->GetX()-shipx)/((range*1000)/30))+30+x);
			ty=(int)(((aiunits[i]->GetY()-shipy)/((range*1000)/30))+30+y);
			boxRGBA(screen, tx-1, ty-1, tx+1, ty+1, r,g,b,255);
			if (i==target)
			{
				rectangleRGBA(screen, tx-3, ty-3, tx+3, ty+3, 0, 255, 0, 255);
			}

		}
	}
	boxRGBA(screen, centx-1, centy-1, centx+1, centy+1, 0,0,255,255);
}
void Map::DisplayOnScreenObjects(int x, int y, int w, int h)
{
	SDL_Surface *screen=SDL_GetVideoSurface();
	int i, blitx, blity, scanx, scany;
	blitx=-10000;
	blity=-10000;
	scanx=(int)((w/2)/zoom);
	scany=(int)((h/2)/zoom);
	for (i=0; i<next; i++) {
		if (objects[i]->IsLoaded())
		{
			if (objects[i]->GetX() < (x+scanx+(objects[i]->GetWidth()/2)/zoom) &&
				objects[i]->GetX() > (x-scanx-(objects[i]->GetWidth()/2)/zoom) &&
				objects[i]->GetY() < (y+scany+(objects[i]->GetHeight()/2)/zoom) &&
				objects[i]->GetY() > (y-scany-(objects[i]->GetHeight()/2)/zoom))
			{
				blitx=(int)(((objects[i]->GetX()-(x-scanx))*zoom)-(objects[i]->GetWidth()/2));
				blity=(int)(((objects[i]->GetY()-(y-scany))*zoom)-(objects[i]->GetHeight()/2));
			}
			if (blitx!=-10000 && blity!=-10000)
			{
				objects[i]->Display(blitx, blity);
			}
		}
		blitx=-10000;
		blity=-10000;
	}
	for (i=0; i<stationnext; i++) {
		if (stations[i]->IsLoaded())
		{
			if (stations[i]->GetX() < (x+scanx+(stations[i]->GetWidth()/2)/zoom) &&
				stations[i]->GetX() > (x-scanx-(stations[i]->GetWidth()/2)/zoom) &&
				stations[i]->GetY() < (y+scany+(stations[i]->GetHeight()/2)/zoom) &&
				stations[i]->GetY() > (y-scany-(stations[i]->GetHeight()/2)/zoom))
			{
				blitx=(int)(((stations[i]->GetX()-(x-scanx))*zoom)-(stations[i]->GetWidth()/2));
				blity=(int)(((stations[i]->GetY()-(y-scany))*zoom)-(stations[i]->GetHeight()/2));
			}
			if (blitx!=-10000 && blity!=-10000)
			{
				stations[i]->Display(blitx, blity);
				if (stations[i]->GetX()-(stations[i]->GetWidth()/2)<=mainship->GetX() &&
					stations[i]->GetY()-(stations[i]->GetHeight()/2)<=mainship->GetY() &&
					stations[i]->GetX()+(stations[i]->GetWidth()/2)>=mainship->GetX() &&
					stations[i]->GetY()+(stations[i]->GetHeight()/2)>=mainship->GetY())
				{
					rectangleRGBA(screen, blitx, blity, blitx+stations[i]->GetWidth(), blity+stations[i]->GetHeight(), 255, 255, 255, 255);
				}
			}
		}
		blitx=-10000;
		blity=-10000;
	}
	for (i=0; i<actnext; i++)
	{
		if (actobjects[i]->IsOrbiter())
		{
			actobjects[i]->Orbit();
		}
		if (actobjects[i]->IsLoaded(actobjects[i]->GetAngle()/5))
		{
			if (actobjects[i]->GetX() < (x+scanx+(actobjects[i]->GetWidth()/2)/zoom) &&
				actobjects[i]->GetX() > (x-scanx-(actobjects[i]->GetWidth()/2)/zoom) &&
				actobjects[i]->GetY() < (y+scany+(actobjects[i]->GetHeight()/2)/zoom) &&
				actobjects[i]->GetY() > (y-scany-(actobjects[i]->GetHeight()/2)/zoom))
			{
				blitx=(int)(((actobjects[i]->GetX()-(x-scanx))*zoom)-(actobjects[i]->GetWidth()/2));
				blity=(int)(((actobjects[i]->GetY()-(y-scany))*zoom)-(actobjects[i]->GetHeight()/2));
			}
			if (blitx!=-10000 && blity!=-10000)
			{
				actobjects[i]->Display(blitx, blity);
			}
		}
		blitx=-10000;
		blity=-10000;
	}
	for (i=0; i<snext; i++)
	{
		if (sobjects[i]->IsLoaded(sobjects[i]->GetAngle()/5))
		{
			if (sobjects[i]->GetX() < (x+scanx+(sobjects[i]->GetWidth()/2)/zoom) &&
				sobjects[i]->GetX() > (x-scanx-(sobjects[i]->GetWidth()/2)/zoom) &&
				sobjects[i]->GetY() < (y+scany+(sobjects[i]->GetHeight()/2)/zoom) &&
				sobjects[i]->GetY() > (y-scany-(sobjects[i]->GetHeight()/2)/zoom))
			{
				blitx=(int)(((sobjects[i]->GetX()-(x-scanx))*zoom)-(sobjects[i]->GetWidth()/2));
				blity=(int)(((sobjects[i]->GetY()-(y-scany))*zoom)-(sobjects[i]->GetHeight()/2));
			}
			if (blitx!=-10000 && blity!=-10000)
			{
				sobjects[i]->Display(blitx, blity);
			}
		}
		blitx=-10000;
		blity=-10000;
	}
	for (i=0; i<aisnext; i++)
	{
		if (aisobjects[i]->IsLoaded(aisobjects[i]->GetAngle()/5))
		{
			if (aisobjects[i]->GetX() < (x+scanx+(aisobjects[i]->GetWidth()/2)/zoom) &&
				aisobjects[i]->GetX() > (x-scanx-(aisobjects[i]->GetWidth()/2)/zoom) &&
				aisobjects[i]->GetY() < (y+scany+(aisobjects[i]->GetHeight()/2)/zoom) &&
				aisobjects[i]->GetY() > (y-scany-(aisobjects[i]->GetHeight()/2)/zoom))
			{
				blitx=(int)(((aisobjects[i]->GetX()-(x-scanx))*zoom)-(aisobjects[i]->GetWidth()/2));
				blity=(int)(((aisobjects[i]->GetY()-(y-scany))*zoom)-(aisobjects[i]->GetHeight()/2));
			}
			if (blitx!=-10000 && blity!=-10000)
			{
				aisobjects[i]->Display(blitx, blity);
			}
		}
		blitx=-10000;
		blity=-10000;
	}
	for (i=0; i<activeanimnext; i++)
	{
		if (activeanim[i] && activeanims[i]->GetCurFrame()!=-1)
		{
			if (activeanims[i]->GetX() < (x+scanx+(activeanims[i]->GetWidth()/2)/zoom) &&
				activeanims[i]->GetX() > (x-scanx-(activeanims[i]->GetWidth()/2)/zoom) &&
				activeanims[i]->GetY() < (y+scany+(activeanims[i]->GetHeight()/2)/zoom) &&
				activeanims[i]->GetY() > (y-scany-(activeanims[i]->GetHeight()/2)/zoom))
			{
				blitx=(int)(((activeanims[i]->GetX()-(x-scanx))*zoom)-(activeanims[i]->GetWidth()/2));
				blity=(int)(((activeanims[i]->GetY()-(y-scany))*zoom)-(activeanims[i]->GetHeight()/2));
			}
			if (blitx!=-10000 && blity!=-10000)
			{
				activeanims[i]->DisplayNext(blitx, blity);
			}
			else activeanims[i]->IncNext();
		}
		if (activeanims[i]->GetCurFrame()==-1)
		{
			this->RemoveFromActiveAnims(i);
		}
		blitx=-10000;
		blity=-10000;
	}
	for (i=0; i<aiunitnext; i++)
	{
		if (aiunits[i]->IsLoaded(aiunits[i]->GetAngle()/5))
		{
			if (aiunits[i]->GetX() < (x+scanx+(aiunits[i]->GetWidth()/2)/zoom) &&
				aiunits[i]->GetX() > (x-scanx-(aiunits[i]->GetWidth()/2)/zoom) &&
				aiunits[i]->GetY() < (y+scany+(aiunits[i]->GetHeight()/2)/zoom) &&
				aiunits[i]->GetY() > (y-scany-(aiunits[i]->GetHeight()/2)/zoom))
			{
				blitx=(int)(((aiunits[i]->GetX()-(x-scanx))*zoom)-(aiunits[i]->GetWidth()/2));
				blity=(int)(((aiunits[i]->GetY()-(y-scany))*zoom)-(aiunits[i]->GetHeight()/2));
				if (i==target)
				{
					int tgx = (int)(((aiunits[i]->GetX()-(x-scanx))*zoom)-(50*zoom));
					int tgy = (int)(((aiunits[i]->GetY()-(y-scany))*zoom)-(50*zoom));
					rectangleRGBA(screen, tgx, tgy, tgx+(100*zoom), tgy+(100*zoom), 0, 255, 0, 255);
				}
			}
			if (blitx!=-10000 && blity!=-10000)
			{
				aiunits[i]->Display(blitx, blity);
			}
		}
		blitx=-10000;
		blity=-10000;
	}
}
void Map::CheckSectors(int sectorx, int sectory)
{
	int i, objectsx, objectsy, aobjectsx, aobjectsy;
	for (i=0; i<next; i++) {
		objectsx=objects[i]->GetSectorX();
		objectsy=objects[i]->GetSectorY();
		if (objectsx <= sectorx+1 && objectsx >= sectorx-1 && objectsy <= sectory+1 && objectsy >= sectory-1)
		{
			if (!objects[i]->IsLoaded()) { objects[i]->LoadImage(); }
		}
		else
		{
			if (objects[i]->IsLoaded()) { objects[i]->FreeImage(); }
		}
	}
	for (i=0; i<stationnext; i++) {
		objectsx=stations[i]->GetSectorX();
		objectsy=stations[i]->GetSectorY();
		if (objectsx <= sectorx+1 && objectsx >= sectorx-1 && objectsy <= sectory+1 && objectsy >= sectory-1)
		{
			if (!stations[i]->IsLoaded()) { stations[i]->LoadImage(); }
		}
		else
		{
			if (stations[i]->IsLoaded()) { stations[i]->FreeImage(); }
		}
	}
	for (i=0; i<actnext; i++) {
		aobjectsx=actobjects[i]->GetSectorX();
		aobjectsy=actobjects[i]->GetSectorY();
		if (aobjectsx <= sectorx+1 && aobjectsx >= sectorx-1 && aobjectsy <= sectory+1 && aobjectsy >= sectory-1)
		{
			if (!actobjects[i]->IsLoaded(actobjects[i]->GetAngle()/5)) { actobjects[i]->LoadImages(0,71); }
		}
		else
		{
			if (actobjects[i]->IsLoaded(actobjects[i]->GetAngle()/5)) { actobjects[i]->FreeImages(); }
		}
	}
	for (i=0; i<aiunitnext; i++) {
		aobjectsx=aiunits[i]->GetSectorX();
		aobjectsy=aiunits[i]->GetSectorY();
		if ((aobjectsx <= sectorx+1 && aobjectsx >= sectorx-1 && aobjectsy <= sectory+1 && aobjectsy >= sectory-1) || i==target)
		{
			if (!aiunits[i]->IsLoaded(aiunits[i]->GetAngle()/5)) { aiunits[i]->LoadImages(0,71); }
		}
		else
		{
			if (aiunits[i]->IsLoaded(aiunits[i]->GetAngle()/5) && i!=target) { aiunits[i]->FreeImages(); }
		}
	}
}
int Map::CheckCollisions(Unit *a)
{
	int i, x, y;
	bitmask *mask1;
	bitmask *mask2;
	SDL_Rect ar, br;
	bool go;
	mask1=a->mask[a->GetAngle()/5];
	ar.x=(int)(a->GetX()-((a->GetWidth()/2)/a->GetZoom()));
	ar.y=(int)(a->GetY()-((a->GetHeight()/2)/a->GetZoom()));
	ar.w=(int)(a->GetWidth()/a->GetZoom());
	ar.h=(int)(a->GetHeight()/a->GetZoom());
	for (i=0; i<actnext; i++)
	{
		if (actobjects[i]->IsLoaded(actobjects[i]->GetAngle()/5))
		{
			br.x=(int)(actobjects[i]->GetX()-((actobjects[i]->GetWidth()/2)/actobjects[i]->GetZoom()));
			br.y=(int)(actobjects[i]->GetY()-((actobjects[i]->GetHeight()/2)/actobjects[i]->GetZoom()));
			br.w=(int)(actobjects[i]->GetWidth()/actobjects[i]->GetZoom());
			br.h=(int)(actobjects[i]->GetHeight()/actobjects[i]->GetZoom());

			go=BoxCollision(br, ar);

			if (go)
			{
				mask2=actobjects[i]->mask[actobjects[i]->GetAngle()/5];
				x=br.x-ar.x;
				y=br.y-ar.y;

				if (bitmask_overlap(mask1,mask2,x,y))
				{
					return i;
				}
			}
		}
	}
	return -1;
}
int Map::CheckAIUnitCollisions(Unit *a)
{
	int i, x, y;
	bitmask *mask1;
	bitmask *mask2;
	SDL_Rect ar, br;
	bool go;
	mask1=a->mask[a->GetAngle()/5];
	ar.x=(int)(a->GetX()-((a->GetWidth()/2)/a->GetZoom()));
	ar.y=(int)(a->GetY()-((a->GetHeight()/2)/a->GetZoom()));
	ar.w=(int)(a->GetWidth()/a->GetZoom());
	ar.h=(int)(a->GetHeight()/a->GetZoom());
	for (i=0; i<aiunitnext; i++)
	{
		if (aiunits[i]->IsLoaded(aiunits[i]->GetAngle()/5))
		{
			br.x=(int)(aiunits[i]->GetX()-((aiunits[i]->GetWidth()/2)/aiunits[i]->GetZoom()));
			br.y=(int)(aiunits[i]->GetY()-((aiunits[i]->GetHeight()/2)/aiunits[i]->GetZoom()));
			br.w=(int)(aiunits[i]->GetWidth()/aiunits[i]->GetZoom());
			br.h=(int)(aiunits[i]->GetHeight()/aiunits[i]->GetZoom());

			go=BoxCollision(br, ar);

			if (go)
			{
				mask2=aiunits[i]->mask[aiunits[i]->GetAngle()/5];
				x=br.x-ar.x;
				y=br.y-ar.y;

				if (bitmask_overlap(mask1,mask2,x,y))
				{
					return i;
				}
			}
		}
	}
	return -1;
}
int Map::CheckShipCollisions(Unit *a)
{
	int x, y;
	bitmask *mask1;
	bitmask *mask2;
	SDL_Rect ar, br;
	bool go;
	mask1=a->mask[a->GetAngle()/5];
	ar.x=(int)(a->GetX()-((a->GetWidth()/2)/a->GetZoom()));
	ar.y=(int)(a->GetY()-((a->GetHeight()/2)/a->GetZoom()));
	ar.w=(int)(a->GetWidth()/a->GetZoom());
	ar.h=(int)(a->GetHeight()/a->GetZoom());
	if (mainship->IsLoaded(mainship->GetAngle()/5))
	{
		br.x=(int)(mainship->GetX()-((mainship->GetWidth()/2)/mainship->GetZoom()));
		br.y=(int)(mainship->GetY()-((mainship->GetHeight()/2)/mainship->GetZoom()));
		br.w=(int)(mainship->GetWidth()/mainship->GetZoom());
		br.h=(int)(mainship->GetHeight()/mainship->GetZoom());

		go=BoxCollision(br, ar);

		if (go)
		{
			if(mainship->HasShield()) mask2=mainship->smask[mainship->GetAngle()/5];
			else mask2=mainship->mask[mainship->GetAngle()/5];
			x=br.x-ar.x;
			y=br.y-ar.y;

			if (bitmask_overlap(mask1,mask2,x,y))
			{
				return 1;
			}
		}
	}
	return 0;
}
void Map::SpawnBullet(int x, int y, int speed, int angle, std::string img, std::string fpk, int aishipindex, int slotid)
{
	if (slotid) mainship->ReduceAmmo(slotid);
	Unit *bullet = new Unit;
	bullet->Init(x,y,img,fpk);
	bullet->SetSpeed(speed);
	bullet->SetAngle(angle);
	bullet->SetZoom(zoom);
	bullet->LoadImages(0,71);
	bullet->SetMMR(255);
	bullet->SetMMG(0);
	bullet->SetMMB(0);
	if (aishipindex!=-1) bullet->SetAI("HEATSEEK");
	this->AddToShootingObjects(bullet, aishipindex);
}
void Map::SpawnAIBullet(int x, int y, int speed, int angle, std::string img, std::string fpk, bool target)
{
	Unit *bullet = new Unit;
	bullet->Init(x,y,img,fpk);
	bullet->SetSpeed(speed);
	bullet->SetAngle(angle);
	bullet->SetZoom(zoom);
	bullet->LoadImages(0,71);
	bullet->SetMMR(255);
	bullet->SetMMG(0);
	bullet->SetMMB(0);
	if (target) bullet->SetAI("HEATSEEK");
	this->AddToAIShootingObjects(bullet);
}
void Map::MoveBullets()
{
	int i, obj;
	for (i=0; i<aiunitnext; i++)
	{
		if (aiunits[i]->IsAIFiring())
		{
			Mix_PlayChannel(0,shot,0);
			this->SpawnAIBullet(aiunits[i]->GetShootX(),aiunits[i]->GetShootY(),aiunits[i]->GetSpeed()+11,aiunits[i]->GetAngle(),"gfx/object/bullet.png","base.fpk");
		}
	}
	for (i=0; i<snext; i++)
	{
		if (SDL_GetTicks()>=s_ticks[i]+5000)
		{
			this->RemoveFromShootingObjects(i);
		}
	}
	for (i=0; i<aisnext; i++)
	{
		if (SDL_GetTicks()>=ais_ticks[i]+5000)
		{
			this->RemoveFromAIShootingObjects(i);
		}
	}
	i=0;
	while(i<snext)
	{
		obj = this->CheckCollisions(sobjects[i]);
		if (obj!=-1)
		{
			if (!strcmp(actobjects[obj]->GetName().c_str(),"asteroid"))
			{
				this->SpawnAnim(0,actobjects[obj]->GetX(), actobjects[obj]->GetY());
				Mix_PlayChannel(1,explode,0);
				mainship->SetCredits(mainship->GetCredits()+actobjects[obj]->GetCredits());
				this->RemoveFromActiveObjects(obj);
				this->RemoveFromShootingObjects(i);
				break;
			}
		}
		obj = this->CheckAIUnitCollisions(sobjects[i]);
		if (obj!=-1)
		{
			this->RemoveFromShootingObjects(i);
			aiunits[obj]->Hit();
			if (aiunits[obj]->GetHullHits()>=aiunits[obj]->GetMaxHullHits())
			{
				this->SpawnAnim(0,aiunits[obj]->GetX(), aiunits[obj]->GetY());
				Mix_PlayChannel(1,explode,0);
				mainship->SetCredits(mainship->GetCredits()+aiunits[obj]->GetCredits());
				this->RemoveFromAIUnits(obj);
				break;
			}
		}
		i++;
	}
	i=0;
	while(i<aisnext)
	{
		obj = this->CheckShipCollisions(aisobjects[i]);
		if (obj)
		{
			mainship->Hit();
			this->RemoveFromAIShootingObjects(i);
			if (mainship->GetHullHits()==mainship->GetMaxHullHits())
			{
				this->SpawnAnim(0,mainship->GetX(), mainship->GetY());
				Mix_PlayChannel(1,explode,0);
				break;
			}
			break;
		}
		i++;
	}
	for (i=0; i<snext; i++)
	{
		int tx=0;
		int ty=0;
		if (shootindex[i]!=-1)
		{
			tx=aiunits[shootindex[i]]->GetX();
			ty=aiunits[shootindex[i]]->GetY();
		}
		sobjects[i]->Move(tx,ty);
	}
	for (i=0; i<aisnext; i++)
	{
		aisobjects[i]->Move(mainship->GetX(),mainship->GetY());
	}
}
void Map::MoveAI(int sx, int sy)
{
	int i;
	for(i=0; i<aiunitnext; i++)
	{
		aiunits[i]->Move(sx,sy);
	}
}
void Map::SetZoom(float szoom)
{
	int i;
	zoom=szoom;
	for (i=0; i<next; i++)
	{
		objects[i]->SetZoom(zoom);
		if (objects[i]->IsLoaded()) objects[i]->LoadImage();
	}
	for (i=0; i<stationnext; i++)
	{
		stations[i]->SetZoom(zoom);
		if (stations[i]->IsLoaded()) stations[i]->LoadImage();
	}
	for (i=0; i<actnext; i++)
	{
		actobjects[i]->SetZoom(zoom);
		if (actobjects[i]->IsLoaded(actobjects[i]->GetAngle()/5)) actobjects[i]->ReloadImages();
	}
	for (i=0; i<snext; i++)
	{
		sobjects[i]->SetZoom(zoom);
		sobjects[i]->ReloadImages();
	}
	for (i=0; i<animnext; i++)
	{
		anims[i]->SetZoom(zoom);
		anims[i]->LoadFrames(anims[i]->GetBase());
	}
	for (i=0; i<activeanimnext; i++)
	{
		activeanims[i]->SetZoom(zoom);
		activeanims[i]->LoadFrames(activeanims[i]->GetBase());
	}
	for (i=0; i<aiunitnext; i++)
	{
		aiunits[i]->SetZoom(zoom);
		if (aiunits[i]->IsLoaded(aiunits[i]->GetAngle()/5)) aiunits[i]->ReloadImages();
	}
}
void Map::AddToAnimations(StaticAnim *anim)
{
	anims[animnext]=anim;
	animnext++;
}
void Map::RemoveFromActiveAnims(int start)
{
	int i;
	//delete activeanims[start];
	for (i=start; i<activeanimnext-1; i++)
	{
		activeanims[i]=activeanims[i+1];
		activeanim[i]=activeanim[i+1];
	}
	activeanimnext-=1;
}
void Map::AddToAIUnits(Unit *unit)
{
	unit->SetZoom(zoom);
	aiunits[aiunitnext]=unit;
	aiunitnext++;
}
void Map::RemoveFromAIUnits(int start)
{
	int i;
	for (i=start; i<aiunitnext-1; i++)
	{
		aiunits[i]=aiunits[i+1];
	}
	aiunitnext-=1;
}
void Map::SpawnAnim(int index, int x, int y)
{
	activeanims[activeanimnext]=anims[index];
	activeanims[activeanimnext]->SetX(x);
	activeanims[activeanimnext]->SetY(y);
	activeanimnext++;
	activeanim[index]=true;
}
int Map::GetAIUnitX(int index)
{
	return aiunits[index]->GetX();
}
int Map::GetAIUnitY(int index)
{
	return aiunits[index]->GetY();
}
void Map::SetTarget(int index)
{
	target=index;
}
void Map::DisplayAITarget(int sx, int sy, int sw, int sh, TTF_Font *font, int targetx, int targety)
{
	SDL_Color white = { 0xFF, 0xFF, 0xFF, 0 };
	SDL_Surface *screen=SDL_GetVideoSurface();
	char disp[300];
	boxRGBA(screen, sx, sy, sx+sw, sy+sh, 128,128,128,164);
	sprintf(disp, "Target Sector: (%d,%d)", (int)targetx/1000, (int)targety/1000);
	drawString(font, sx+5, sy, disp, white);
	if (target!=-1)
	{
		int shipx=sx+((sw-aiunits[target]->GetDispImageW())/2);
		int shipy=sy+((sh-aiunits[target]->GetDispImageH())/2);
		aiunits[target]->DisplayDispImage(shipx,shipy);
	}
	else centerString(font, sx+(sw/2), sy+(sh/2), "No Target", white);
}
int Map::GetAIUnitImageW(int index)
{
	return aiunits[index]->GetDispImageW();
}
int Map::GetAIUnitImageH(int index)
{
	return aiunits[index]->GetDispImageH();
}
void Map::Dock(int x, int y)
{
	int i;
	for (i=0; i<stationnext; i++)
	{
		if (stations[i]->IsLoaded())
		{
			if (stations[i]->GetX()-(stations[i]->GetWidth()/2)<=x &&
				stations[i]->GetY()-(stations[i]->GetHeight()/2)<=y &&
				stations[i]->GetX()+(stations[i]->GetWidth()/2)>=x &&
				stations[i]->GetY()+(stations[i]->GetHeight()/2)>=y)
			{
				stations[i]->Dock(mainship);
			}
		}
	}
}
int Map::GetObjCount()
{
	return actnext;
}
int Map::GetAICount()
{
	return aiunitnext;
}

